// Copyright 2004, FreeHEP.
package hep.wired.heprep.edit;

import hep.wired.edit.GraphicsPanelEdit;
import hep.wired.edit.WiredEdit;
import hep.wired.heprep.graphicspanel.HepRepPanel;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import javax.swing.undo.UndoableEdit;

/**
 * Generic Edit for changing the GraphicsMode.
 * 
 * @author Mark Donszelmann
 * @version $Id: GraphicsModeEdit.java 8598 2006-08-14 20:38:02Z duns $
 */

public class GraphicsModeEdit extends WiredEdit implements GraphicsPanelEdit {

    private String name;

    private Object oldValue;

    private Object value;

    public GraphicsModeEdit(String name) {
        this(name, null);
    }

    public GraphicsModeEdit(String name, boolean value) {
        this(name, new Boolean(value));
    }

    public GraphicsModeEdit(String name, double value) {
        this(name, new Double(value));
    }

    public GraphicsModeEdit(String name, int value) {
        this(name, new Integer(value));
    }

    private GraphicsModeEdit(String name, Object value) {
        super();
        this.name = name;
        this.value = value;
        this.oldValue = null;
    }

    public WiredEdit copy(RecordPlot plot) {
        WiredEdit copy = new GraphicsModeEdit(name, value);
        copy.setRecordPlot(plot);
        return copy;
    }

    public String getPresentationName() {
        return "GraphicsModeEdit( " + name + " = " + value + " )";
    }

    public boolean isSupportedBy(GraphicsPanel p) {
        return p instanceof HepRepPanel;
    }

    public boolean addEdit(UndoableEdit edit) {
        if (edit instanceof GraphicsModeEdit) {
            // only if they are equal
            GraphicsModeEdit modeEdit = (GraphicsModeEdit) edit;
            if (modeEdit.name.equals(name)) {
                return modeEdit.value.equals(value);
            }
        }
        return false;
    }

    public void redoEdit() {
        HepRepPanel panel = (HepRepPanel) getRecordPlot().getGraphicsPanel();
        oldValue = panel.getGraphicsMode().getValue(name);
        panel.getGraphicsMode().setValue(name, value);
        getRecordPlot().repaint();
    }

    public void undoEdit() {
        HepRepPanel panel = (HepRepPanel) getRecordPlot().getGraphicsPanel();
        value = panel.getGraphicsMode().getValue(name);
        panel.getGraphicsMode().setValue(name, oldValue);
        getRecordPlot().repaint();
    }
}
