// Copyright 2003-2005, FreeHEP.
package hep.wired.heprep.edit;

import hep.wired.edit.GraphicsPanelEdit;
import hep.wired.edit.WiredEdit;
import hep.wired.heprep.feature.NamedVariables;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.util.logging.Logger;

import javax.swing.undo.UndoableEdit;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: SetVariable.java 8598 2006-08-14 20:38:02Z duns $
 */
public class SetVariable extends WiredEdit implements GraphicsPanelEdit {

    private static Logger logger = Logger.getLogger(SetVariable.class
            .getPackage().getName());

    private String name;

    private double v;

    private double previousV;

    /**
     * Creates a variable edit.
     */
    public SetVariable(String name) {
        this(name, 0);
    }

    /**
     * Creates a variable edit.
     */
    public SetVariable(String name, double v) {
        super();
        this.name = name;
        this.v = v;
    }

    public WiredEdit copy(RecordPlot plot) {
        WiredEdit copy = new SetVariable(name, v);
        copy.setRecordPlot(plot);
        return copy;
    }

    public String getPresentationName() {
        StringBuffer s = new StringBuffer();
        s.append(toString());
        s.append(" ");
        s.append(name);
        s.append("=");
        s.append(format.format(v));
        return s.toString();
    }

    public String toString() {
        return "SetVariable";
    }

    public boolean addEdit(UndoableEdit edit) {
        if (edit instanceof SetVariable) {
            SetVariable setVariable = (SetVariable) edit;
            if (name.equals(setVariable.name)) {
                v = setVariable.v;
                return true;
            }
        }
        return false;
    }

    public boolean isSupportedBy(GraphicsPanel p) {
        NamedVariables variables = (NamedVariables) p
                .getFeature(NamedVariables.class);
        if (variables != null) {
            return (variables.exists(name));
        }
        return false;
    }

    protected void redoEdit() {
        GraphicsPanel panel = getRecordPlot().getGraphicsPanel();
        NamedVariables variables = (NamedVariables) panel
                .getFeature(NamedVariables.class);
        if ((variables != null) && variables.exists(name)) {
            previousV = variables.getDoubleVariable(name);
            variables.setVariable(name, v);
        } else {
            System.err.println("Could not redo: " + this);
            return;
        }
        getRecordPlot().repaint();
        logger.finer(toString());
    }

    protected void undoEdit() {
        GraphicsPanel panel = getRecordPlot().getGraphicsPanel();
        NamedVariables variables = (NamedVariables) panel
                .getFeature(NamedVariables.class);
        if ((variables != null) && variables.exists(name)) {
            variables.setVariable(name, previousV);
        } else {
            System.err.println("Could not undo: " + this);
            return;
        }
        getRecordPlot().repaint();
        logger.finer(toString());
    }

    public String getName() {
        return name;
    }
}
