package hep.wired.heprep.graphicspanel;

import java.awt.Component;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;

import hep.wired.edit.FitToWindow;
import hep.wired.edit.Reset;
import hep.wired.edit.Translate;
import hep.wired.heprep.edit.GraphicsModeEdit;
import hep.wired.heprep.image.WiredImage;
import hep.wired.heprep.interaction.ClickToSetLocation;
import hep.wired.heprep.interaction.SetLocationPanel;
import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.services.InteractionHandler;
import hep.wired.services.RecordPlot;
import hep.wired.util.WiredUndoManager;
import org.freehep.application.Application;
import org.freehep.application.PrintHelper;
import org.freehep.application.studio.Studio;
import org.freehep.util.commanddispatcher.BooleanCommandState;
import org.freehep.util.commanddispatcher.CommandProcessor;
import org.freehep.util.commanddispatcher.CommandState;
import org.freehep.graphicsbase.util.export.ExportDialog;
import org.freehep.graphicsbase.util.export.VectorGraphicsTransferable;

/**
 * Defines a command handler for plot commands.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: HepRepPanelCommandHandler.java 14047 2012-10-22 20:35:37Z onoprien $
 */
public class HepRepPanelCommandHandler extends CommandProcessor implements UndoableEditListener {

// -- Private parts : ----------------------------------------------------------

  private RecordPlot plot;
  private WiredPlugin plugin;
  private WiredUndoManager undoManager = new WiredUndoManager();

// -- Construction : -----------------------------------------------------------

  /** Creates a command handler attached to the plot. */
  public HepRepPanelCommandHandler(RecordPlot plot) {
    super();
    this.plot = plot;
    plugin = WiredPlugin.getPlugin();
  }

  public void enableSaveAs(CommandState state) {
    state.setEnabled(plot.getRecord() != null);
    setChanged();
  }
  
// -- Selecting interaction handlers : -----------------------------------------

  public void enableDragToScale(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.ZOOM_DRAG, state);
  }

  public void onDragToScale(boolean on) {
    onInteractionHandler(WiredPlugin.IH.ZOOM_DRAG, on);
  }

  public void enableDragRectangleToScale(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.ZOOM_RECTANGLE, state);
  }

  public void onDragRectangleToScale(boolean on) {
    onInteractionHandler(WiredPlugin.IH.ZOOM_RECTANGLE, on);
  }

  public void enableDragAngledRectangleToScale(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.ZOOM_ANGLE, state);
  }

  public void onDragAngledRectangleToScale(boolean on) {
    onInteractionHandler(WiredPlugin.IH.ZOOM_ANGLE, on);
  }

  public void enableDragBallToRotate(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.ROTATE_BALL, state);
  }

  public void onDragBallToRotate(boolean on) {
    onInteractionHandler(WiredPlugin.IH.ROTATE_BALL, on);
  }

  public void enableDragToRotate(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.ROTATE_FLAT, state);
  }

  public void onDragToRotate(boolean on) {
    onInteractionHandler(WiredPlugin.IH.ROTATE_FLAT, on);
  }

  public void enableDragToTranslate(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.TRANSLATE, state);
  }

  public void onDragToTranslate(boolean on) {
    onInteractionHandler(WiredPlugin.IH.TRANSLATE, on);
  }

  public void enableDragToMeasure(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.MEASURE, state);
  }

  public void onDragToMeasure(boolean on) {
    onInteractionHandler(WiredPlugin.IH.MEASURE, on);
  }

  public void enableHoverToPick(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.PICK_HOVER, state);
  }

  public void onHoverToPick(boolean on) {
    onInteractionHandler(WiredPlugin.IH.PICK_HOVER, on);
  }

  public void enableDragRectangleToPick(BooleanCommandState state) {
    enableInteractionHandler(WiredPlugin.IH.PICK_RECTANGLE, state);
  }

  public void onDragRectangleToPick(boolean on) {
    onInteractionHandler(WiredPlugin.IH.PICK_RECTANGLE, on);
  }

// -- Miscellaneous : ----------------------------------------------------------

  public void onSaveAs() {
    Studio studio = (Studio) Application.getApplication();
    Properties user = studio.getUserProperties();
    String creator = user.getProperty("fullVersion");

    String fileName = "Wired_" + plot.getName().replaceAll(" ", "_");

    ExportDialog dialog = new ExportDialog(creator, true);
    if (plot.getGraphicsPanel() instanceof HepRepPanel) {
      dialog.addExportFileType(new XMLHepRepExportFileType());
    }
    dialog.setUserProperties(user);
    dialog.showExportDialog(studio, "Export...", (Component) plot, fileName);
  }

  public void enableSetTranslation(CommandState state) {
    state.setEnabled(plot.supports(new ClickToSetLocation()));
    setChanged();
  }

  public void onSetTranslation() {
    final JButton selectButton = new JButton("Select Location");
    final JButton cancelButton = new JButton("Cancel");
    SetLocationPanel panel = new SetLocationPanel(plot, selectButton);

    final JOptionPane pane = new JOptionPane(panel,
            JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION,
            WiredImage.getIcon("Translate3D32"), new Object[]{
              selectButton, cancelButton});

    selectButton.addActionListener(new ActionListener() {

      public void actionPerformed(ActionEvent event) {
        pane.setValue(selectButton);
      }
    });

    cancelButton.addActionListener(new ActionListener() {

      public void actionPerformed(ActionEvent event) {
        pane.setValue(cancelButton);
      }
    });

    JDialog dialog = pane.createDialog(((JComponent) plot).getTopLevelAncestor(), "Select 3D position");
    dialog.setResizable(true);
    dialog.setVisible(true);
    dialog.dispose();

    Object value = pane.getValue();
    if ((value != null) && value.equals(selectButton)) {
      try {
        double[] xyz = panel.getXYZ();
        System.out.println(xyz[0] + " " + xyz[1] + " " + xyz[2]);
        plot.postEdit(new Translate(-xyz[0], -xyz[1], -xyz[2]));
      } catch (UnsupportedOperationException e) {
        JOptionPane.showMessageDialog(((JComponent) plot).getTopLevelAncestor(), e.getMessage(),
                "Operation not supported", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  public void enableDrawWideLines(BooleanCommandState state) {
    state.setEnabled(plot.supports(new GraphicsModeEdit("drawWideLines")));
    state.setSelected(((HepRepGraphicsMode) ((HepRepPanel) plot.getGraphicsPanel()).getGraphicsMode()).useLineWidth);
    setChanged();
  }

  public void onDrawWideLines(boolean on) {
    plot.postEdit(new GraphicsModeEdit("drawWideLines", on));
    setChanged();
  }

  public void enableFillBoxes(BooleanCommandState state) {
    state.setEnabled(plot.supports(new GraphicsModeEdit("fillBoxes")));
    state.setSelected(((HepRepGraphicsMode) ((HepRepPanel) plot.getGraphicsPanel()).getGraphicsMode()).fillBoxes);
    setChanged();
  }

  public void onFillBoxes(boolean on) {
    plot.postEdit(new GraphicsModeEdit("fillBoxes", on));
    setChanged();
  }

  public void enableDrawFrames(BooleanCommandState state) {
    state.setEnabled(plot.supports(new GraphicsModeEdit("drawFrames")));
    state.setSelected(((HepRepGraphicsMode) ((HepRepPanel) plot.getGraphicsPanel()).getGraphicsMode()).drawFrames);
    setChanged();
  }

  public void onDrawFrames(boolean on) {
    plot.postEdit(new GraphicsModeEdit("drawFrames", on));
    setChanged();
  }

  public void enableAntiAlias(BooleanCommandState state) {
    // Bug 4485298 (Sun) creates an Exception when rendering large lines
    // with
    // antialising switched on. Fixed for 1.5.0-beta-2.
    // state.setEnabled(false);
    state.setEnabled(plot.supports(new GraphicsModeEdit("antiAlias")));
    state.setSelected(((HepRepGraphicsMode) ((HepRepPanel) plot.getGraphicsPanel()).getGraphicsMode()).antiAlias);
    setChanged();
  }

  public void onAntiAlias(boolean on) {
    plot.postEdit(new GraphicsModeEdit("antiAlias", on));
    setChanged();
  }

  public void enableUseLayering(BooleanCommandState state) {
    state.setEnabled(plot.supports(new GraphicsModeEdit("useLayering")));
    state.setSelected(((HepRepGraphicsMode) ((HepRepPanel) plot.getGraphicsPanel()).getGraphicsMode()).useLayering);
    setChanged();
  }

  public void onUseLayering(boolean on) {
    plot.postEdit(new GraphicsModeEdit("useLayering", on));
    setChanged();
  }

  // =========================================================================================
  // Additional handling, normally handled by WiredPlotCommandHandler, but
  // plugin specific...
  // =========================================================================================
  public void enableRedo(CommandState state) {
    state.setEnabled(undoManager.canRedo());
    state.setText("Redo " + undoManager.getRedoString());
    state.setToolTipText(undoManager.getRedoPresentationName());
    setChanged();
  }

  public void onRedo() {
    undoManager.redo();
    setChanged();
  }

  public void enableUndo(CommandState state) {
    state.setEnabled(undoManager.canUndo());
    state.setText("Undo " + undoManager.getUndoString());
    state.setToolTipText(undoManager.getUndoPresentationName());
    setChanged();
  }

  public void onUndo() {
    undoManager.undo();
    setChanged();
  }

  public void onPrintSetup() {
    Studio studio = (Studio) Application.getApplication();

    PrintHelper printHelper = new PrintHelper((Component) plot, studio);
    printHelper.showOptionsDialog(studio);
  }

  public void enablePrint(CommandState state) {
    state.setEnabled(plot.getRecord() != null);
    setChanged();
  }

  public void onPrint() throws Exception {
    Studio studio = (Studio) Application.getApplication();

    PrintHelper printHelper = new PrintHelper((Component) plot, studio);
    printHelper.print();
  }

  public void enablePrintPreview(CommandState state) {
    state.setEnabled(plot.getRecord() != null);
    setChanged();
  }

  public void onPrintPreview() throws Exception {
    Studio studio = (Studio) Application.getApplication();

    PrintHelper printHelper = new PrintHelper((Component) plot, studio);
    printHelper.printPreview(studio);
  }

  public void enableCopy(CommandState state) {
    state.setEnabled(plot.getRecord() != null);
    setChanged();
  }

  public void onCopy() {
    Clipboard clipBoard = Toolkit.getDefaultToolkit().getSystemClipboard();
    VectorGraphicsTransferable transferable = new VectorGraphicsTransferable(
            (Component) plot);
    clipBoard.setContents(transferable, transferable);
  }

  public void undoableEditHappened(UndoableEditEvent edit) {
    undoManager.addEdit(edit.getEdit());
    setChanged();
  }

  public void enableReset(CommandState state) {
    state.setEnabled(plot.supports(new Reset()));
    setChanged();
  }

  public void onReset() {
    plot.postEdit(new Reset());
  }

  public void enableFitToWindow(CommandState state) {
    state.setEnabled(plot.supports(new FitToWindow()));
    setChanged();
  }

  public void onFitToWindow() {
    plot.postEdit(new FitToWindow());
  }

// -- Local methods : ----------------------------------------------------------

  private void enableInteractionHandler(WiredPlugin.IH handler, BooleanCommandState state) {
    InteractionHandler ih = handler.get();
    state.setEnabled(plot.supports(ih));
    state.setSelected(plugin.getInteractionHandler() == ih);
    setChanged();
  }

  private void onInteractionHandler(WiredPlugin.IH handler, boolean on) {
    if (on) {
      InteractionHandler ih = handler.get();
      if (plugin.getInteractionHandler() == ih) {
        plugin.setControlPanel(handler.control());
      } else {
        plugin.setInteractionHandler(ih);
      }
    }
    setChanged();
  }

}
