// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.graphicspanel;

import hep.graphics.heprep.HepRep;
import hep.graphics.heprep.HepRepFactory;
import hep.graphics.heprep.HepRepWriter;
import hep.graphics.heprep.xml.XMLHepRepFactory;
import hep.wired.plot.WiredPlot;

import java.awt.Component;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Properties;

import org.freehep.graphicsbase.util.export.ExportFileType;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: XMLHepRepExportFileType.java 14047 2012-10-22 20:35:37Z onoprien $
 */
public class XMLHepRepExportFileType extends ExportFileType {

    private HepRepFactory factory;

    public XMLHepRepExportFileType() {
        factory = new XMLHepRepFactory();
    }

    public String getDescription() {
        return "HepRep2";
    }

    public String[] getExtensions() {
        // WIRED-454
        return new String[] { "heprep.zip", "heprep.gz", "heprep" /*
                                                                     * ,
                                                                     * "bheprep.zip",
                                                                     * "bheprep.gz",
                                                                     * "bheprep"
                                                                     */};
    }

    public String[] getMIMETypes() {
        return new String[] { "application/x-heprep" };
    }

    public File adjustFilename(File file, Properties user) {
        return super.adjustFilename(file, user);
        /*
         * UserProperties options = new UserProperties(user,
         * SVGGraphics2D.getDefaultProperties()); if
         * (options.isProperty(SVGGraphics2D.COMPRESS)) { return
         * adjustExtension(file, "svgz", null); } else { return
         * adjustExtension(file, "svg", null); }
         */
    }

    public void exportToFile(OutputStream os, Component[] target,
            Component parent, Properties properties, String creator)
            throws IOException {
        HepRepWriter writer = factory.createHepRepWriter(os, target.length > 1,
                target.length > 1);
        write(writer, target, "heprep");
    }

    public void exportToFile(File file, Component[] target, Component parent,
            Properties properties, String creator) throws IOException {
        String name = file.getName().toLowerCase();
        boolean random = name.endsWith(".zip");
        boolean compress = random || name.endsWith(".gz");
        String type = name.endsWith(".bheprep.zip")
                || name.endsWith(".bheprep.gz") || name.endsWith(".bheprep") ? "bheprep"
                : "heprep";
        // WIRED-454
        type = "heprep";
        HepRepWriter writer = factory.createHepRepWriter(new FileOutputStream(
                file), random, compress);
        write(writer, target, type);
    }

    private void write(HepRepWriter writer, Component[] target, String type)
            throws IOException {
        for (int i = 0; i < target.length; i++) {
            if (target[i] instanceof WiredPlot) {
                WiredPlot plot = (WiredPlot) target[i];
                HepRep heprep = (HepRep) plot.getRecord();
                if (heprep != null)
                    writer.write(heprep, plot.getName() + "-" + i + "." + type);
            }
        }
        writer.close();
    }
}
