package hep.wired.heprep.interaction;

import java.awt.BorderLayout;
import java.util.*;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;

import hep.graphics.heprep.HepRep;
import hep.graphics.heprep.HepRepAttributeAdapter;
import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.HepRepIterator;
import hep.graphics.heprep.HepRepType;
import hep.graphics.heprep.util.HepRepUtil;
import hep.wired.cut.BooleanConditionDefinition;
import hep.wired.cut.Condition;
import hep.wired.cut.ConditionDefinition;
import hep.wired.cut.ConditionEditor;
import hep.wired.cut.Cut;
import hep.wired.cut.CutEditor;
import hep.wired.cut.CutManager;
import hep.wired.cut.CutSet;
import hep.wired.cut.CutSetEditor;
import hep.wired.cut.CutSetEditor.ActionNew;
import hep.wired.cut.DoubleConditionDefinition;
import hep.wired.cut.DoubleSliderEditor;
import hep.wired.cut.IntegerConditionDefinition;
import hep.wired.cut.IntegerSliderEditor;
import hep.wired.cut.SliderPanel;
import hep.wired.cut.StringConditionDefinition;
import hep.wired.cut.StringConditionEditor;
import hep.wired.heprep.graphicspanel.ControlPanel;
import hep.wired.heprep.graphicspanel.HepRepPanel;
import hep.wired.heprep.graphicspanel.HepRepPanelModel;
import hep.wired.heprep.image.WiredImage;
import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.heprep.tree.HepRepTree;
import hep.wired.heprep.tree.HepRepTreeModel;
import hep.wired.plot.WiredPlot;
import hep.wired.util.TitledPanel;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import javax.swing.BorderFactory;
import javax.swing.JSplitPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.freehep.application.Application;

/**
 * {@link ControlPanel} that allows user to view and manipulate cuts.
 * 
 * @author Dmitry Onoprienko
 * @version $Id: CutPanel.java 14753 2013-05-03 18:27:34Z onoprien $
 * 
 */
public class CutPanel extends ControlPanel implements CutManager {

// -- Private parts : ----------------------------------------------------------
  
  private Properties _user;

  private HepRepPanelModel _plotModel;  // model of the currently displayed plot
  private HepRepCut _cut;               // currently selected cut

  private HepRep _heprep;                                    // currently loaded HepRep
  private LinkedHashMap<String, ConditionDefinition> _defs;  // known ConditionDefinitions
  private SortedSet<String> _currentDefs;                    // names of attributes present in currently loaded HepRep

  private JPanel _mainPanel;
  private JPanel _stringEditorPanel;
  private SliderPanel _sliderPanel;
  private HepRepTree _typeTree;

  private CutSetEditor _cutSetEditor;
  private CutEditor _cutEditor;

  private final int _hspace = 3;
  private final Dimension _hDim = new Dimension(_hspace, 0);
  private final int _vspace = 2;
  private final Dimension _vDim = new Dimension(0, _vspace);


// -- Construction and initialization : ----------------------------------------

  public CutPanel() {

    super("Cuts", "Add or Change Cuts", WiredImage.getIcon("Scissors%w", 16), WiredPlugin.IH.TRANSLATE.get());

    _user = Application.getApplication().getUserProperties();
    _defs = new LinkedHashMap();

    setLayout(new BorderLayout());
    _mainPanel = new JPanel();
    _mainPanel.setLayout(new BoxLayout(_mainPanel,BoxLayout.Y_AXIS));
    add(_mainPanel, BorderLayout.CENTER);

    _cutSetEditor = new CutSetEditor();
    _cutSetEditor.setActionNew(_cutSetEditor.new ActionNew() {
      public void actionPerformed(ActionEvent e) {
        _cutSetEditor.add(createCut());
      }
    });

    Box box = Box.createHorizontalBox();
    box.add(_cutSetEditor.getEnableBox());
    box.add(Box.createRigidArea(_hDim));
    box.add(_cutSetEditor.getInvertBox());
    box.add(Box.createRigidArea(_hDim));
    box.add(_cutSetEditor.getPassThroughBox());
    box.add(Box.createHorizontalGlue());
    _mainPanel.add(box);
    _mainPanel.add(Box.createRigidArea(_vDim));

    TitledPanel titledPanel = new TitledPanel("Cuts", _cutSetEditor.getCutTable(), _cutSetEditor.getActionsButton());
    titledPanel.setBorder(BorderFactory.createEtchedBorder());
    _mainPanel.add(titledPanel);

    JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    splitPane.setResizeWeight(.4);
    splitPane.setAlignmentX(CENTER_ALIGNMENT);

    _typeTree = new HepRepTree();
    titledPanel = new TitledPanel("From Types:", _typeTree);
    titledPanel.setBorder(BorderFactory.createEtchedBorder());
    splitPane.setTopComponent(titledPanel);
//    _mainPanel.add(titledPanel);

    _cutSetEditor.getCutTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
       public void valueChanged(ListSelectionEvent e) {
         if (_cut != null && _cut.getTypes() == null) _cut.setTypesTree(null);
         _cut = (HepRepCut) _cutSetEditor.getSelectedCut();
         if (_cut == null) {
           _typeTree.setModel(null);
         } else {
           HepRepTreeModel treeModel = _cut.getTypeTree();
           if (treeModel == null) {
             treeModel = new HepRepTreeModel(_heprep, true);
             _cut.setTypesTree(treeModel);
           }
           _typeTree.setModel(treeModel);
         }
        }
    });

    _cutEditor = _cutSetEditor.getCutEditor();
    _cutEditor.setManager(this);
    StringConditionEditor edit = new StringConditionEditor();
    _stringEditorPanel = edit.getPanel();
    _cutEditor.setConditionEditor(ConditionDefinition.Type.STRING, edit);
    _sliderPanel = new SliderPanel();
    _cutEditor.setConditionEditor(ConditionDefinition.Type.INTEGER, new IntegerSliderEditor(_sliderPanel));
    _cutEditor.setConditionEditor(ConditionDefinition.Type.DOUBLE, new DoubleSliderEditor(_sliderPanel));
    titledPanel = new TitledPanel("Conditions", _cutEditor.getConditionsTable(),
            new Component[] {_cutEditor.getHideDisabledBox(), _cutEditor.getActionsButton()});
    titledPanel.setBorder(BorderFactory.createEtchedBorder());

    splitPane.setBottomComponent(titledPanel);
    _mainPanel.add(splitPane);

  }

// -- Responding to plugin-level events : --------------------------------------

  /** Called when this panel is selected or deselected. */
  @Override
  public void setSelected(boolean selected) {
    super.setSelected(selected);
    if (selected) {
      WiredPlugin plugin = WiredPlugin.getPlugin();
      if (_heprep != plugin.getRecord()) setRecord(plugin.getRecord());
      WiredPlot p =  plugin.getPlot();
      if (p == null) return;
      HepRepPanelModel model = ((HepRepPanel)(p.getGraphicsPanel())).getModel();
      if (_plotModel != model) {
        _plotModel = model;
        CutSet cutSet = _plotModel.getCutSet();
        if (cutSet == null) {
          cutSet = createCutSet();
          _plotModel.setCutSet(cutSet);
        }
        _cutSetEditor.setCutSet(cutSet);
      }
    }
  }


  /**
   * Called when a new record is loaded into WIRED plugin.
   * Resets and updates domains of all ConditionDefinitions.
   * Compiles a set of attribute names in the current heprep.
   */
  @Override
  public void setRecord(HepRep heprep) {

    _heprep = heprep;
    if (heprep == null) return;

    _currentDefs = new TreeSet<String>();
    for (ConditionDefinition cd : _defs.values()) {
      cd.setValueIsAdjusting(true);
      cd.setDomain(null);
    }
    HepRepIterator it = HepRepUtil.getInstances(heprep.getInstanceTreeList(), null, null, false);
    it.addHepRepAttributeListener(null, new HepRepAttributeAdapter() {
      public void setAttribute(HepRepInstance instance, String key, boolean value, int showLabel) {
        _currentDefs.add(key);
        ConditionDefinition cd = _defs.get(key);
        if (cd == null) {
          _defs.put(key, new BooleanConditionDefinition(key));
        } else if (cd.getType() != ConditionDefinition.Type.BOOLEAN) {
          _defs.put(key, new BooleanConditionDefinition(key));
          removeConDef(cd);
        }
      }
      public void setAttribute(HepRepInstance instance, String key, double value, int showLabel) {
//        if (key.equals("ch")) {
//          System.out.println("");
//          System.out.println("Att: "+ value +" + "+ (instance.getAttValueFromNode(key) == null ? ("T: "+instance.getAttValue(key)) : instance.getAttValueFromNode(key)));
//        }
        _currentDefs.add(key);
        ConditionDefinition cd = _defs.get(key);
        if (cd == null) {
          DoubleConditionDefinition newcd = new DoubleConditionDefinition(key);
          _defs.put(key, newcd);
          newcd.addToDomain(value);
        } else if (cd.getType() != ConditionDefinition.Type.DOUBLE) {
          DoubleConditionDefinition newcd = new DoubleConditionDefinition(key);
          _defs.put(key, newcd);
          newcd.addToDomain(value);
          removeConDef(cd);
        } else {
          ((DoubleConditionDefinition)cd).addToDomain(value);
        }
      }
      public void setAttribute(HepRepInstance instance, String key, int value, int showLabel) {
        _currentDefs.add(key);
        ConditionDefinition cd = _defs.get(key);
        if (cd == null) {
          IntegerConditionDefinition newcd = new IntegerConditionDefinition(key);
          _defs.put(key, newcd);
          newcd.addToDomain(value);
        } else if (cd.getType() != ConditionDefinition.Type.INTEGER) {
          IntegerConditionDefinition newcd = new IntegerConditionDefinition(key);
          _defs.put(key, newcd);
          newcd.addToDomain(value);
          removeConDef(cd);
        } else {
          ((IntegerConditionDefinition)cd).addToDomain(value);
        }
      }
      public void setAttribute(HepRepInstance instance, String key, String value, int showLabel) {
        _currentDefs.add(key);
        ConditionDefinition cd = _defs.get(key);
        if (cd == null) {
          StringConditionDefinition newcd = new StringConditionDefinition(key);
          _defs.put(key, newcd);
          newcd.addToDomain(value);
        } else if (cd.getType() != ConditionDefinition.Type.STRING) {
          StringConditionDefinition newcd = new StringConditionDefinition(key);
          _defs.put(key, newcd);
          newcd.addToDomain(value);
          removeConDef(cd);
        } else {
          ((StringConditionDefinition)cd).addToDomain(value);
        }
      }
    });
    while (it.hasNext()) {
      HepRepInstance instance = it.nextInstance();
    }
    for (ConditionDefinition cd : _defs.values()) cd.setValueIsAdjusting(false);

    ArrayList<HepRepCut> cuts = new ArrayList(_cutSetEditor.getCuts());
    for (HepRepCut cut : cuts) {
      try {
        cut.setRecord(heprep);
      } catch (IllegalArgumentException x) {
        _cutSetEditor.delete(cut);
      }
    }
  }

// -- Implementing CutManager : ------------------------------------------------

  @Override
  public void showConditionEditor(ConditionEditor editor) {
    if (editor == null) {
      try {
        remove(1);
      } catch (ArrayIndexOutOfBoundsException x) {}
    } else {
      if (editor.getType() == ConditionDefinition.Type.STRING) {
        add(_stringEditorPanel, BorderLayout.SOUTH);
      } else if (editor.getType() == ConditionDefinition.Type.INTEGER || editor.getType() == ConditionDefinition.Type.DOUBLE) {
        add(_sliderPanel, BorderLayout.EAST);
      }
    }
    validate();
  }

  @Override
  public Collection<ConditionDefinition> getDefinitions() {
    return Collections.unmodifiableCollection(_defs.values());
  }

  @Override
  public ConditionDefinition getDefinition(String name) {
    return _defs.get(name);
  }

  @Override
  public Set<ConditionDefinition> getCurrentDefinitions() {
    if (_currentDefs == null) return Collections.emptySet();
    LinkedHashSet<ConditionDefinition> out = new  LinkedHashSet();
    for (String name : _currentDefs) out.add(_defs.get(name));
    return out;
  }

  @Override
  public Set<ConditionDefinition> getCurrentDefinitions(Cut cut) {
    if (cut == null) return getCurrentDefinitions();
    final Set<HepRepType> types = ((HepRepCut)cut).getTypes();
    if (types == null) return getCurrentDefinitions();
    if (_heprep == null) return Collections.emptySet();
    HepRepIterator it = HepRepUtil.getInstances(_heprep.getInstanceTreeList(), null, null, false);
    final TreeSet<String> names = new TreeSet();
    it.addHepRepAttributeListener(null, new HepRepAttributeAdapter() {
      public void setAttribute(HepRepInstance instance, String key, boolean value, int showLabel) {
        if (types.contains(instance.getType())) names.add(key);
      }
      public void setAttribute(HepRepInstance instance, String key, double value, int showLabel) {
        if (types.contains(instance.getType())) names.add(key);
      }
      public void setAttribute(HepRepInstance instance, String key, int value, int showLabel) {
        if (types.contains(instance.getType())) names.add(key);
      }
      public void setAttribute(HepRepInstance instance, String key, String value, int showLabel) {
        if (types.contains(instance.getType())) names.add(key);
      }
    });
    while (it.hasNext()) it.nextInstance();
    LinkedHashSet<ConditionDefinition> out = new LinkedHashSet();
    for (String name : names) out.add(_defs.get(name));
    return out;
  }


// -- Local methods : ----------------------------------------------------------

  private CutSet createCutSet() {
    return new CutSet();
  }

  private HepRepCut createCut() {
    HepRepCut cut = new HepRepCut();
    for (String name : _currentDefs) {
      Condition c = _defs.get(name).createCondition();
      cut.add(c);
    }
    return cut;
  }

  /** Removes all cuts/conditions that use the specified definition. */
  private void removeConDef(ConditionDefinition cd) {
    cd.remove();
  }

// -----------------------------------------------------------------------------
}
