package hep.wired.heprep.interaction;

import hep.wired.heprep.services.InverseProjection;
import hep.wired.interaction.DefaultInteractionHandler;
import hep.wired.services.RecordPlot;
import hep.wired.services.ViewPort;
import hep.wired.util.XYZindices;
import java.awt.event.MouseEvent;
import javax.swing.JLabel;
import org.freehep.application.Application;
import org.freehep.util.ScientificFormat;

/**
 * Base class for WIRED interaction handlers. <p> Adds functionality to the
 * {@link DefaultInteractionHandler} : <ul> <li>Displays mouse position in
 * projections where this is possible.</li> </ul>
 *
 * @version $Id: $
 * @author onoprien
 */
public class DefaultWiredInteractionHandler extends DefaultInteractionHandler implements XYZindices {
  
  private final double _tolerance = 0.001;
  static private JLabel _posLabel;
  protected ScientificFormat _format = new ScientificFormat();

  protected DefaultWiredInteractionHandler(String name) {
    super(name);
  }

  /**
   * Called when underlying plot is selected/unselected.
   */
  public void setSelected(RecordPlot plot, boolean selected) {
    displayPosition("");
  }

  public void mouseExited(RecordPlot plot, MouseEvent event) {
    displayPosition("");
  }

  public void mouseMoved(RecordPlot plot, MouseEvent event) {
    try {
      ViewPort viewPort = plot.getGraphicsPanel().getViewPort();
      InverseProjection projection = (InverseProjection) plot.getGraphicsPanel().getFeature(InverseProjection.class);
      double u = event.getX();
      double v = event.getY();
      double p1[] = new double[]{u, v, 0.};
      double p2[] = new double[]{u, v, 1.};
      p1 = viewPort.inverseTransform(p1);
      p2 = viewPort.inverseTransform(p2);
      p1 = projection.inverseTransform(p1);
      p2 = projection.inverseTransform(p2);
      StringBuilder sb = new StringBuilder();
      int n = 0;
      if (Math.abs(p1[X] - p2[X]) < _tolerance) {
        sb.append(" X = ").append(_format.format(p1[X])).append(" ");
        n++;
      }
      if (Math.abs(p1[Y] - p2[Y]) < _tolerance) {
        sb.append(" Y = ").append(_format.format(p1[Y])).append(" ");
        n++;
      }
      if (Math.abs(p1[Z] - p2[Z]) < _tolerance) {
        sb.append(" Z = ").append(_format.format(p1[Z])).append(" ");
        n++;
      }
      displayPosition(n<3 ? sb.toString() : "");
    } catch (UnsupportedOperationException x) {
      displayPosition("");
    }
  }
  
  private void displayPosition(String position) {
    if (_posLabel == null) {
      _posLabel = new JLabel();
      _posLabel.setAlignmentX(0.5f);
      Application.getApplication().getStatusBar().add(_posLabel);
    }
    _posLabel.setText(position);
  }
  
 }
