package hep.wired.heprep.interaction;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.ListSelectionModel;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import hep.graphics.heprep.HepRepInstance;
import hep.wired.heprep.graphicspanel.ControlPanel;
import hep.wired.heprep.graphicspanel.HepRepPanel;
import hep.wired.heprep.graphicspanel.HepRepPanelModel;
import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.heprep.tree.HepRepTree;
import hep.wired.heprep.tree.HepRepTreeModel;
import hep.wired.heprep.util.Keys;
import hep.wired.image.WiredBaseImage;
import hep.wired.services.InteractionHandler;
import hep.wired.services.RecordPlot;
import hep.wired.util.OptionButton;
import hep.wired.util.ScientificTable;
import hep.wired.util.TitledPanel;
import hep.wired.util.tree.WiredTreeModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.freehep.swing.layout.TableLayout;

/**
 * Panel to show HepRep attribute information.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: HepRepInfoPanel.java 14746 2013-05-02 18:59:56Z onoprien $
 */
public class HepRepInfoPanel extends ControlPanel implements ChangeListener {

// -- Private parts : ----------------------------------------------------------

  // Plot :

  private transient RecordPlot _plot;
  private HepRepPanelModel _plotModel;

  // Handler selection panel :

  private ButtonGroup _group;
  private JPanel _shapePanel;
  private Map<InteractionHandler, JToggleButton> _buttonsByHandler;
  private JToggleButton _deselect;

  // Layer panel - what is pickable

  private LayerTableModel _layerTableModel;
  private TitledPanel _layerPanel;

  // Type panel - picked objects of what types are shown in instance table and highlighted in view

  private TitledPanel _typePanel;
  private HepRepTree _typeTree;

  // Category panel - attributes of what categories are shown in attribute table

  private CategoryTableModel _categoryTableModel;
  private TitledPanel _categoryPanel;

  // Instance table - picked instances

  private InstanceTableModel _instanceTableModel;
  private JTable _instanceTable;
  private TitledPanel _instancePanel;
  private ListSelectionModel _instanceSelectionModel;
  private ListSelectionListener _instanceSelectionListener;

  // Attribute table - attributes of selected instance

  private AttributeTableModel _attributeTableModel;
  private JTable _attributeTable;
  private TitledPanel _attributePanel;

  // Other components

  private JButton zoomButton;
  private JButton translateButton;
  private JCheckBox pickWhileDragging;

  private transient PickHandler interactionHandler;


// -- Constructor : ------------------------------------------------------------

  public HepRepInfoPanel() {

    super("Info", "Information on Picked Objects", WiredBaseImage.getIcon("PickInfo%w", 16), HoverToPick.getInstance());
    setLayout(new TableLayout());

    // Layer panel - model

    _layerTableModel = new LayerTableModel(Collections.<Flag>emptyList()) {
      protected void changed() {
        if (_plotModel != null) {
          _plotModel.fireEvent(HepRepPanelModel.Event.Type.PICK_OPTIONS);
        }
      }
    };

    // Category panel - model

    _categoryTableModel = new CategoryTableModel(Collections.<Flag>emptyList()) {
      protected void changed() {
        _attributeTableModel.setShownCategories(_plotModel.getAttCategories());
        _attributeTableModel.update();
        repaint();
      }
    };

    // Instance table

    _instanceTableModel = new InstanceTableModel();
    _instanceTable = new ScientificTable(_instanceTableModel);
    _instanceSelectionModel = _instanceTable.getSelectionModel();
    _instanceSelectionModel.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    _instanceSelectionListener = new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent event) {
        if (_plotModel != null) {
          if (event.getValueIsAdjusting()) return;
          int[] sel = _instanceTable.getSelectedRows();
          ArrayList<HepRepInstance> selection = new ArrayList(sel.length);
          for (int i : sel) {
            selection.add(_instanceTableModel.get(i));
          }
          _plotModel.setSelection(selection, HepRepInfoPanel.this);
          _attributeTableModel.setInstance(selection.size() == 1 ? selection.get(0) : null);
          _attributeTableModel.update();
          _attributePanel.setTitle("Attributes of selected object ("+ _attributeTableModel.getRowCount() + "):");
          updateButtons();
        }
      }
    };

    // Attribute table - model

    _attributeTableModel = new AttributeTableModel(Collections.<Flag>emptyList());
    _attributeTable = new ScientificTable(_attributeTableModel);

    // Handler selection panel - View :

    String insets = " [3 3 3 3] ";

    _group = new ButtonGroup();
    _deselect = new JToggleButton("deselect");
    _group.add(_deselect);
    _buttonsByHandler = new LinkedHashMap();
    _shapePanel = new JPanel();
    _shapePanel.setLayout(new TableLayout());
    _shapePanel.setBorder(new TitledBorder(new EtchedBorder(), "Shape"));
    add(_shapePanel, "0 *" + insets + "w");

    // Options panel - contains layer and type panels

    final JPanel optionsPanel = new JPanel();
    optionsPanel.setName("hep.wired.InstanceOptions");
    optionsPanel.setLayout(new TableLayout());

    _layerPanel = new TitledPanel("Not set", new ScientificTable(_layerTableModel));
    optionsPanel.add(_layerPanel, "0 *" + insets + "wh");

    _typeTree = new HepRepTree();
//    _typeMarkListener = new WiredTreeModel.MarkListener() {
//      public void markChanged(MarkEvent e) {
//        interactionHandler.update(_plot);
//      }
//    };
    _typePanel = new TitledPanel("Show instances from types:", _typeTree);
    optionsPanel.add(_typePanel, "0 *" + insets + "wh");

    _categoryPanel = new TitledPanel("Not set", new ScientificTable(_categoryTableModel));
    _categoryPanel.setName(Keys.pickingAttributeOptions);

    // Instance and attribute panels

    OptionButton instanceButton = new OptionButton("Options...", "Instance Table Options", optionsPanel, this);
    _instancePanel = new TitledPanel("Not set", _instanceTable, instanceButton);
    _instancePanel.setName(Keys.pickingInstanceOptions);

    OptionButton attributeButton = new OptionButton("Options...","Attribute Table Options", _categoryPanel, this);
    _attributePanel = new TitledPanel("Not set", _attributeTable, attributeButton);

    JSplitPane dataPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    dataPanel.setTopComponent(_instancePanel);
    dataPanel.setBottomComponent(_attributePanel);
    dataPanel.setResizeWeight(0.30);

    // Actions/Settings panel

    zoomButton = new JButton("Zoom into Region");
    zoomButton.setEnabled(false);
    zoomButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        interactionHandler.zoomIntoRegion(_plot);
        zoomButton.setEnabled(interactionHandler.isRegionZoomable());
      }
    });

    translateButton = new JButton("Translate to Picked Object");
    translateButton.setEnabled(false);
    translateButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        interactionHandler.translateToPicked(_plot);
        translateButton.setEnabled(interactionHandler.isPickedTranslateable(_plot));
      }
    });

    pickWhileDragging = new JCheckBox("Pick while Moving/Dragging");
    pickWhileDragging.setEnabled(false);
    pickWhileDragging.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        interactionHandler.setPickWhileDragging(pickWhileDragging.isSelected());
        pickWhileDragging.setEnabled(interactionHandler.canSetPickWhileDragging());
      }
    });

    JPanel selectionPanel = new JPanel();
    selectionPanel.setBorder(new TitledBorder(new EtchedBorder(),"Actions / Settings"));
    selectionPanel.setLayout(new TableLayout());
    selectionPanel.add(zoomButton, "0 * ltw");
    selectionPanel.add(translateButton, "0 * ltw");
    selectionPanel.add(pickWhileDragging, "0 * ltw");

    add(selectionPanel, "0 *" + insets + "w");
    add(dataPanel, "0 *" + insets + "wh");
  }

  /**
   * Adds interaction handler to this HepRepInfoPanel. A button for the specified handler
   * will be displayed in the handler selection panel. Choosing the handler will set it
   * as an interaction handler for the WiredPlugin instance.
   */
  public void addHandler(final InteractionHandler h, boolean selected) {
    if (h != null) {
      final JToggleButton b = new JToggleButton(h.getIcon(16), selected);
      b.setToolTipText(h.getDescription());
      b.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          WiredPlugin.getPlugin().setInteractionHandler(h);
        }
      });
      _group.add(b);
      _shapePanel.add(b, "* * lt");
      _buttonsByHandler.put(h, b);
    } else {
      // FIXME table got to have a better way
      _shapePanel.add(new JLabel(), "* * ltW");
    }
  }


// -- Handler setters : --------------------------------------------------------

  /**
   * Sets PickHandler instance associated with this HepRepInfoPanel.
   * Called by PickInteractionHandlers when mouse enters plot - not clear what for...
   */
  public void setInteractionHandler(PickHandler interactionHandler, RecordPlot plot) {
    this.interactionHandler = interactionHandler;
    updateButtons();
  }

  /**
   * Associates the specified <tt>InteractionHandler</tt> with this panel.
   * If the supplied <tt>InteractionHandler</tt> is one those that have been previously
   * added through a call to {@link #addHandler} method, the corresponding button is
   * selected; if not, all buttons are deselected.
   */
  public void setInteractionHandler(InteractionHandler h) {
    super.setInteractionHandler(h);
    if (h instanceof PickHandler) interactionHandler = (PickHandler) h;
    if (h == null) {
      _deselect.setSelected(true);
    } else {
      JToggleButton b = (JToggleButton) _buttonsByHandler.get(h);
      if (b != null && b.isEnabled()) {
        b.setSelected(true);
        _defaultInteractionHandler = h;
      } else {
        _deselect.setSelected(true);
      }
    }
    updateButtons();
  }


// -- Listening to HepRepPanelModel : ------------------------------------------

  /**
   * Handles update from the plot model.
   * Current tasks: the method is called when the lists of picked or selected instances are updated.
   */
  public void stateChanged(ChangeEvent event) {
    if (event.getSource() != this) {
      setInstances(_plotModel.getSelection(), _plotModel.getPick());
      updateButtons();
    }
  }


// -- Initialization when the panel is selected : ------------------------------

  /** Called when this panel is selected or deselected. */
  public void setSelected(boolean selected) {
    super.setSelected(selected);

    if (selected) {

      // looking up current plot

      _plot = WiredPlugin.getPlugin().getPlot();
      if (_plot == null) return;

      // setting to a new plot

      try {
        _plotModel = ((HepRepPanel) _plot.getGraphicsPanel()).getModel();
      } catch (ClassCastException x) {
        return;
      }

      _layerTableModel.setKeys(_plotModel.getPickableLayers());
      _layerTableModel.fireTableDataChanged();
      _layerPanel.setTitle("Pick from layers (" + _layerTableModel.getRowCount() + "):");
      _layerPanel.revalidate();
      _layerPanel.repaint();

      HepRepTreeModel treeModel = _plotModel.getPickableTypesModel();
      _typeTree.setModel(treeModel, true);

      List<Flag> categories = _plotModel.getAttCategories();
      _categoryTableModel.setKeys(categories);
      _attributeTableModel.setShownCategories(categories);
      
      setInstances(_plotModel.getSelection(), _plotModel.getPick());
      _plotModel.addListener(this, HepRepPanelModel.Event.Type.SELECTION);

      repaint();

    } else { // deselecting

      if (_plotModel != null) {
        _plotModel.removeListener(this);
      }

    }
  }
  
  
// -- Updaters : ---------------------------------------------------------------

  private void setInstances(Set<HepRepInstance> selection, Collection<HepRepInstance> picked) {
    Set<HepRepInstance> pick = Collections.newSetFromMap(new IdentityHashMap<HepRepInstance, Boolean>());
    pick.addAll(picked);
    pick.addAll(selection);
    ArrayList<HepRepInstance> pickList = new ArrayList(pick);
    _instanceSelectionModel.removeListSelectionListener(_instanceSelectionListener);
    _instanceTableModel.setKeys(pickList);
    _instanceTableModel.fireTableDataChanged();
    for (int i = 0; i < pickList.size(); i++) {
      if (selection.contains(pickList.get(i))) _instanceSelectionModel.addSelectionInterval(i, i);
    }
    _instancePanel.setTitle("Picked objects ("+ _instanceTableModel.getRowCount() + "):");
    _instanceSelectionModel.addListSelectionListener(_instanceSelectionListener);
    _attributeTableModel.setInstance(selection.size() == 1 ? selection.iterator().next() : null);
    _attributeTableModel.update();
    _attributePanel.setTitle("Attributes of selected object ("+ _attributeTableModel.getRowCount() + "):");
  }

  private void updateButtons() {
    if (interactionHandler != null) {
      zoomButton.setEnabled(interactionHandler.isRegionZoomable());
      translateButton.setEnabled(interactionHandler.isPickedTranslateable(_plot));
      pickWhileDragging.setSelected(interactionHandler.isPickWhileDragging());
      pickWhileDragging.setEnabled(interactionHandler.canSetPickWhileDragging());
    } else {
      zoomButton.setEnabled(false);
      translateButton.setEnabled(false);
      pickWhileDragging.setSelected(false);
      pickWhileDragging.setEnabled(false);
    }
  }

}
