package hep.wired.heprep.interaction;

import java.util.*;

import hep.graphics.heprep.HepRepInstance;
import hep.wired.edit.SetTranslate;
import hep.wired.heprep.graphicspanel.HepRepPanel;
import hep.wired.heprep.graphicspanel.HepRepPanelModel;
import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.heprep.util.WiredHepRepUtil;
import hep.wired.interaction.DefaultInteractionHandler;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;
import hep.wired.util.XYZindices;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.freehep.application.Application;

/**
 * Base class for interaction handlers that pick objects on a WIRED plot..
 *
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: PickInteractionHandler.java 14746 2013-05-02 18:59:56Z onoprien $
 */
public abstract class PickInteractionHandler extends DefaultWiredInteractionHandler implements PickHandler, XYZindices, ChangeListener {

// -- Private parts : ----------------------------------------------------------

  protected transient HepRepInstance currentInstance;
  protected transient Set pickedInstances = Collections.newSetFromMap(new IdentityHashMap());
  protected transient Set selectedInstances = Collections.newSetFromMap(new IdentityHashMap());
  protected boolean pickWhileDragging;

// -- Construction : -----------------------------------------------------------

  public PickInteractionHandler(String name, boolean pickWhileDragging) {
    super(name);
    this.pickWhileDragging = pickWhileDragging;
  }

  @Deprecated
  protected HepRepInfoPanel info() {
    return WiredPlugin.getPlugin().getHepRepInfoPanel();
  }

// -- Implementing InteractionHandler : ----------------------------------------

  public boolean isSupportedBy(GraphicsPanel panel) {
    return panel instanceof HepRepPanel;
  }

  public void setRecord(RecordPlot plot, Object record) {
    update(plot);
  }

  public void setSelected(RecordPlot plot, boolean selected) {
    if (selected) {
      Application.getApplication().setStatusMessage(getDescription());
      try {
        ((HepRepPanel)plot.getGraphicsPanel()).getModel().addListener(this, HepRepPanelModel.Event.Type.PICK_OPTIONS);
      } catch (ClassCastException x) {}
    } else {
      Application.getApplication().setStatusMessage("");
      clearShape(plot);
      try {
        ((HepRepPanel)plot.getGraphicsPanel()).getModel().removeListener(this);
      } catch (ClassCastException x) {}
    }
  }

// -- Implementing PickHandler : -----------------------------------------------

  /** Informs the plot of new set of picked instances. */
  public void setSelection(RecordPlot plot, Set<HepRepInstance> selectedInstances) {
    GraphicsPanel panel = plot.getGraphicsPanel();
    if (panel instanceof HepRepPanel) {
      ((HepRepPanel) panel).getModel().setPick(selectedInstances, this);
    }
  }

  /**
   * Performs picking action based on current cursor location if possible,
   * decorates the plot and informs it of newly picked objects.
   * This class provides empty implementation.
   */
  public void update(RecordPlot plot) {}

  public boolean canSetPickWhileDragging() {
    return true;
  }

  public void setPickWhileDragging(boolean state) {
    pickWhileDragging = state;
  }

  public boolean isPickWhileDragging() {
    return pickWhileDragging;
  }

//  public boolean isPickedTranslateable() {
//    return selectedInstances.size() == 1;
//  }

  public boolean isPickedTranslateable(RecordPlot plot) {
    return plot != null && ((HepRepPanel)plot.getGraphicsPanel()).getModel().getSelection().size() == 1;
  }

  public void translateToPicked(RecordPlot plot) {
    if (plot == null) return;
    Set<HepRepInstance> selection = ((HepRepPanel)plot.getGraphicsPanel()).getModel().getSelection();
    if (selection.size() != 1) return;
//    if (!isPickedTranslateable()) return;

    double[] t = WiredHepRepUtil.getCenterPoint((HepRepInstance) selection.iterator().next());
//    double[] t = WiredHepRepUtil.getCenterPoint((HepRepInstance) selectedInstances.iterator().next());
    if (t != null) {
      plot.postEdit(new SetTranslate(-t[X], -t[Y], -t[Z]));
      clearShape(plot);
    }
  }

  protected abstract void clearShape(RecordPlot plot);

// -- Listening to plot model : ---------------------------------------

  /** Updates plot in response to picking options changes. */
  public void stateChanged(ChangeEvent e) {
    update(WiredPlugin.getPlugin().getPlot());
  }


}