// Copyright 2003-2006, FreeHEP.
package hep.wired.heprep.interaction;

import hep.wired.interaction.NullInteractionHandler;
import hep.wired.services.DelegatingInteractionHandler;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.InteractionHandler;
import hep.wired.services.RecordPlot;

import java.awt.Component;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.Icon;

import org.freehep.xml.io.XMLIOManager;
import org.jdom.Element;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: RoutingInteractionHandler.java 8598 2006-08-14 20:38:02Z duns $
 */
public abstract class RoutingInteractionHandler implements
        DelegatingInteractionHandler {

    private String name;

    protected List/* <InteractionHandler> */handlers;

    protected InteractionHandler interactionHandler;

    /**
     * Creates a DelegatingInteractionHandler.
     */
    public RoutingInteractionHandler(String name) {
        this.name = name;
    }

    public void add(InteractionHandler h) {
        add(h, handlers == null);
    }

    public void add(final InteractionHandler h, boolean selected) {
        if (handlers == null)
            handlers = new ArrayList();
        handlers.add(h);
        if (selected)
            interactionHandler = h;
    }

    public void setInteractionHandler(InteractionHandler interactionHandler) {
        if (interactionHandler == null) {
            this.interactionHandler = NullInteractionHandler.getInstance();
        } else {
            this.interactionHandler = interactionHandler;
        }
    }

    // 
    // DelegatingInteractionHandler
    //
    public InteractionHandler getInteractionHandler() {
        return interactionHandler;
    }

    //
    // InteractionHandler
    //
    public Icon getIcon(int size) {
        return interactionHandler.getIcon(size);
    }

    public boolean isSupportedBy(GraphicsPanel panel) {
        if (interactionHandler == NullInteractionHandler.getInstance())
            return true;

        for (Iterator i = handlers.iterator(); i.hasNext();) {
            InteractionHandler handler = (InteractionHandler) i.next();
            if (handler.isSupportedBy(panel))
                return true;
        }
        return false;
    }

    public void changeCursor(RecordPlot plot, InputEvent event) {
        interactionHandler.changeCursor(plot, event);
    }

    public String getID() {
        return interactionHandler.getName();
    }

    public void setRecord(RecordPlot plot, Object record) {
        interactionHandler.setRecord(plot, record);
    }

    public void setSelected(RecordPlot plot, boolean selected) {
        // FIXME ???
        // WiredPlugin.getPlugin().setControl(getControl());
        interactionHandler.setSelected(plot, selected);
    }

    public void setSize(RecordPlot plot, int width, int height) {
        interactionHandler.setSize(plot, width, height);
    }

    public void reset(RecordPlot plot, InputEvent event) {
        interactionHandler.reset(plot, event);
    }

    public void mouseButton1Clicked(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseButton1Clicked(plot, event);
    }

    public void mouseButton1Pressed(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseButton1Pressed(plot, event);
    }

    public void mouseButton1Released(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseButton1Released(plot, event);
    }

    public void mouseButton1DragStarted(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseButton1DragStarted(plot, event);
    }

    public void mouseButton1Dragged(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseButton1Dragged(plot, event);
    }

    public void mouseButton1DragEnded(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseButton1DragEnded(plot, event);
    }

    public void mouseMoved(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseMoved(plot, event);
    }

    public void mouseEntered(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseEntered(plot, event);
    }

    public void mouseExited(RecordPlot plot, MouseEvent event) {
        interactionHandler.mouseExited(plot, event);
    }

    public void mouseWheelMoved(RecordPlot plot, MouseWheelEvent event) {
        interactionHandler.mouseWheelMoved(plot, event);
    }

    public boolean escKeyPressed(RecordPlot plot, KeyEvent event) {
        return interactionHandler.escKeyPressed(plot, event);
    }

    public boolean escKeyReleased(RecordPlot plot, KeyEvent event) {
        return interactionHandler.escKeyReleased(plot, event);
    }

    public boolean altKeyPressed(RecordPlot plot, KeyEvent event) {
        return interactionHandler.altKeyPressed(plot, event);
    }

    public boolean altKeyReleased(RecordPlot plot, KeyEvent event) {
        return interactionHandler.altKeyReleased(plot, event);
    }

    public boolean ctrlKeyPressed(RecordPlot plot, KeyEvent event) {
        return interactionHandler.ctrlKeyPressed(plot, event);
    }

    public boolean ctrlKeyReleased(RecordPlot plot, KeyEvent event) {
        return interactionHandler.ctrlKeyReleased(plot, event);
    }

    public boolean shiftKeyPressed(RecordPlot plot, KeyEvent event) {
        return interactionHandler.shiftKeyPressed(plot, event);
    }

    public boolean shiftKeyReleased(RecordPlot plot, KeyEvent event) {
        return interactionHandler.shiftKeyReleased(plot, event);
    }

    public boolean metaKeyPressed(RecordPlot plot, KeyEvent event) {
        return interactionHandler.metaKeyPressed(plot, event);
    }

    public boolean metaKeyReleased(RecordPlot plot, KeyEvent event) {
        return interactionHandler.metaKeyReleased(plot, event);
    }

    public boolean ctrlMetaKeyPressed(RecordPlot plot, KeyEvent event) {
        return interactionHandler.ctrlMetaKeyPressed(plot, event);
    }

    public boolean ctrlMetaKeyReleased(RecordPlot plot, KeyEvent event) {
        return interactionHandler.ctrlMetaKeyReleased(plot, event);
    }

    public boolean otherKeyPressed(RecordPlot plot, KeyEvent event) {
        return interactionHandler.otherKeyPressed(plot, event);
    }

    public boolean otherKeyReleased(RecordPlot plot, KeyEvent event) {
        return interactionHandler.otherKeyReleased(plot, event);
    }

    public boolean keyTyped(RecordPlot plot, KeyEvent event) {
        return interactionHandler.keyTyped(plot, event);
    }

    public int getWidth() {
        return interactionHandler.getWidth();
    }

    public int getHeight() {
        return interactionHandler.getHeight();
    }

    public Component getControl() {
        return null;
    }

    public String getName() {
        return name;
    }
        
    //
    // XMLIO
    //
    public void save(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
        nodeEl.setAttribute("name", name);
        if (handlers != null) {
            for (Iterator i = handlers.iterator(); i.hasNext();) {
                InteractionHandler handler = (InteractionHandler) i.next();
                nodeEl.addContent(xmlioManager.save(handler));
                if (handler == interactionHandler)
                    nodeEl.setAttribute("selected", handler.getName());
            }
        }
    }

    public void restore(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
        name = nodeEl.getAttributeValue("name");
        String selected = nodeEl.getAttributeValue("selected");
        for (Iterator i = nodeEl.getChildren().iterator(); i.hasNext();) {
            InteractionHandler handler = (InteractionHandler) xmlioManager
                    .restore((Element) i.next());
            add(handler, handler.getName().equals(selected));
        }
    }
}
