// Copyright 2004-2006, FreeHEP.
package hep.wired.heprep.interaction;

import hep.graphics.heprep.HepRep;
import hep.wired.edit.Rotate;
import hep.wired.edit.Scale;
import hep.wired.feature.HasBoundingBox;
import hep.wired.heprep.services.InverseProjection;
import hep.wired.services.RecordPlot;
import hep.wired.viewport.RectangularViewPort;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.geom.Rectangle2D;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import org.freehep.swing.layout.TableLayout;

/**
 * Panel to specify a 3D location in an UV and its 90 degree rotated WV and UW
 * projections.
 * 
 * @author Mark Donszelmann
 * @version $Id: SetLocationPanel.java 8598 2006-08-14 20:38:02Z duns $
 */
public class SetLocationPanel extends JPanel {

    private static final double popupSizeFactor = 0.8;

    private JButton selectButton;

    private RecordPlot plot;

    private RecordPlot uvPlot, wvPlot, uwPlot;

    private double uSize, vSize, wSize;

    private int u = -1;

    private int v = -1;

    private int w = -1;

    private boolean uIsFixed = false;

    private boolean vIsFixed = false;

    private boolean wIsFixed = false;

    public SetLocationPanel(RecordPlot plot, JButton select) {
        this.plot = plot;
        this.selectButton = select;
        selectButton.setEnabled(false);

        setLayout(new TableLayout());

        // setup table
        final AbstractTableModel tableModel = new AbstractTableModel() {

            private String[] columnNames = { "Name", "Value", "Fixed" };

            private String[] rowNames = { "U", "V", "W" };

            public String getColumnName(int col) {
                return columnNames[col];
            }

            public int getColumnCount() {
                return columnNames.length;
            }

            public int getRowCount() {
                return rowNames.length;
            }

            public boolean isCellEditable(int row, int col) {
                return false;
            }

            public Class getColumnClass(int c) {
                switch (c) {
                case 0:
                    return String.class;
                case 1:
                    return Integer.class;
                case 2:
                    return Boolean.class;
                default:
                    return String.class;
                }
            }

            public Object getValueAt(int row, int col) {
                switch (col) {
                case 0:
                    return rowNames[row];
                case 1:
                    return new Integer(row == 0 ? u : row == 1 ? v : w);
                case 2:
                    return new Boolean(row == 0 ? uIsFixed
                            : row == 1 ? vIsFixed : wIsFixed);
                default:
                    return "";
                }
            }
        };
        JTable table = new JTable(tableModel);
        // install special renderer to show "not set"
        table.setDefaultRenderer(Integer.class, new DefaultTableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value, boolean isSelected, boolean hasFocus,
                    int row, int column) {
                if ((column == 1) && (((Integer) value).intValue() == -1)) {
                    return new JLabel("Not set");
                }
                return super.getTableCellRendererComponent(table, value,
                        isSelected, hasFocus, row, column);
            }
        });

        JPanel tablePanel = new JPanel();
        tablePanel.setLayout(new BorderLayout());
        tablePanel.add(table.getTableHeader(), BorderLayout.NORTH);
        tablePanel.add(table, BorderLayout.CENTER);

        // FIXME WIRED-388 needs clone
        // FIXME WIRED-388 needs explanation text in html?
        HepRep heprep = (HepRep) plot.getRecord();

        uvPlot = plot.copy("UV", heprep, false);
        wvPlot = plot.copy("WV", heprep, false);
        uwPlot = plot.copy("UW", heprep, false);

        final ClickToSetLocation uv = new ClickToSetLocation();
        uvPlot.setInteractionHandler(uv);

        final ClickToSetLocation wv = new ClickToSetLocation();
        wvPlot.setInteractionHandler(wv);
        wvPlot.postEdit(new Rotate(Math.PI / 2, 0, 1, 0));

        final ClickToSetLocation uw = new ClickToSetLocation();
        uwPlot.setInteractionHandler(uw);
        uwPlot.postEdit(new Rotate(Math.PI / 2, 1, 0, 0));

        uv.addLocationListener(new LocationListener() {
            public void setLocation(RecordPlot plot, int newU, int newV,
                    boolean fixU, boolean fixV) {
                u = newU;
                uIsFixed = fixU;
                v = newV;
                vIsFixed = fixV;
                wv.setLocation(wvPlot, w, v, wIsFixed, vIsFixed);
                uw.setLocation(uwPlot, u, w, uIsFixed, wIsFixed);
                selectButton.setEnabled(uIsFixed && vIsFixed && wIsFixed);
                tableModel.fireTableRowsUpdated(0, tableModel.getRowCount());
            }
        });

        wv.addLocationListener(new LocationListener() {
            public void setLocation(RecordPlot plot, int newW, int newV,
                    boolean fixW, boolean fixV) {
                w = newW;
                wIsFixed = fixW;
                v = newV;
                vIsFixed = fixV;
                uv.setLocation(uvPlot, u, v, uIsFixed, vIsFixed);
                uw.setLocation(uwPlot, u, w, uIsFixed, wIsFixed);
                selectButton.setEnabled(uIsFixed && vIsFixed && wIsFixed);
                tableModel.fireTableRowsUpdated(0, tableModel.getRowCount());
            }
        });

        uw.addLocationListener(new LocationListener() {
            public void setLocation(RecordPlot plot, int newU, int newW,
                    boolean fixU, boolean fixW) {
                u = newU;
                uIsFixed = fixU;
                w = newW;
                wIsFixed = fixW;
                uv.setLocation(uvPlot, u, v, uIsFixed, vIsFixed);
                wv.setLocation(wvPlot, w, v, wIsFixed, vIsFixed);
                selectButton.setEnabled(uIsFixed && vIsFixed && wIsFixed);
                tableModel.fireTableRowsUpdated(0, tableModel.getRowCount());
            }
        });

        // Find out the bounding boxes
        Rectangle2D rectangle;
        rectangle = ((HasBoundingBox) uvPlot.getGraphicsPanel().getFeature(
                HasBoundingBox.class)).getBoundingBoxForPlot();
        System.out.println(rectangle);
        uSize = (rectangle == null) ? 500 : rectangle.getWidth();
        vSize = (rectangle == null) ? 500 : rectangle.getHeight();
        rectangle = ((HasBoundingBox) wvPlot.getGraphicsPanel().getFeature(
                HasBoundingBox.class)).getBoundingBoxForPlot();
        System.out.println(rectangle);
        wSize = (rectangle == null) ? 500 : rectangle.getWidth();

        // calculate scale factors to fit plots
        // FIXME WIRED-388 cast!
        RectangularViewPort viewPort = (RectangularViewPort) plot
                .getGraphicsPanel().getViewPort();
        uvPlot.postEdit(new Scale(Math.max(viewPort.getWidth() / uSize,
                viewPort.getHeight() / vSize)));
        wvPlot.postEdit(new Scale(Math.max(viewPort.getWidth() / wSize,
                viewPort.getHeight() / vSize)));
        uwPlot.postEdit(new Scale(Math.max(viewPort.getWidth() / uSize,
                viewPort.getHeight() / wSize)));

        // setup layout correctly
        String insets = " [3 3 3 3]";
        add(new JLabel(), "0 *");
        add(new JLabel("U"), "1 *" + insets);
        add(new JLabel("W"), "2 *" + insets);

        add(new JLabel("V"), "0 * [1 1 1 1]");
        add((Component) uvPlot, "1 *" + insets + " {" + (int) uSize + ", "
                + (int) vSize + "} wh");
        add((Component) wvPlot, "2 *" + insets + " {" + (int) wSize + ", "
                + (int) vSize + "} wh");

        add(new JLabel("W"), "0 *" + insets);
        add((Component) uwPlot, "1 *" + insets + " {" + (int) uSize + ", "
                + (int) wSize + "} wh");
        add(tablePanel, "2 *" + insets + " {" + (int) wSize + ", "
                + (int) wSize + "} wh");
    }

    /**
     * Returns the chosen XYZ location.
     */
    public double[] getXYZ() throws UnsupportedOperationException {
        double xyz[] = new double[3];
        xyz[0] = xyz[1] = xyz[2] = 0;

        /*
         * FIXME WIRED-388 // Depending on if u, v and/or w are set, calculate
         * via inverse viewports the // xyz that belong to it. This translation
         * is with respect to the viewports. if (u != -1) { if (v != -1) { // u
         * and v known ViewPort uvPort =
         * uvPlot.getGraphicsPanel().getViewPort(); xyz[0] =
         * uvPort.inverseTransformX(u, v); xyz[1] = uvPort.inverseTransformY(u,
         * v); if (w != -1) { // u, v, w known xyz[2] =
         * wvPlot.getGraphicsPanel().getViewPort().inverseTransformX(w, v); } }
         * else if (w != -1) { // u and w known ViewPort uwPort =
         * uwPlot.getGraphicsPanel().getViewPort(); xyz[0] =
         * uwPort.inverseTransformX(u, w); xyz[2] = uwPort.inverseTransformY(u,
         * w); } } else { if ((v != -1) && (w != -1)) { // v and w known
         * ViewPort wvPort = wvPlot.getGraphicsPanel().getViewPort(); xyz[2] =
         * wvPort.inverseTransformX(w, v); xyz[1] = wvPort.inverseTransformY(w,
         * v); } }
         */
        // run the translation through the inverse delta of the uvPlot, since
        // that one
        // may have a different scale than the original plot.
        xyz = ((InverseProjection) uvPlot.getGraphicsPanel().getFeature(
                InverseProjection.class)).inverseDeltaTransform(xyz);

        // The resulting translate needs to be run through the delta projection
        // of the original
        // plot to get a transform relative to that scale factor.
        return ((InverseProjection) plot.getGraphicsPanel().getFeature(
                InverseProjection.class)).deltaTransform(xyz);
    }

    public Dimension getPreferredSize() {
        Dimension d = ((JComponent) plot).getTopLevelAncestor().getSize();
        d.width *= popupSizeFactor;
        d.height *= popupSizeFactor;
        double ratio = (uSize + wSize) / (vSize + wSize);
        d = (ratio > 1) ? new Dimension((int) d.getWidth(),
                (int) (d.getWidth() / ratio)) : new Dimension((int) (d
                .getHeight() * ratio), (int) d.getHeight());
        return d;
    }

    public void setBounds(int x, int y, int width, int height) {
        // FIXME WIRED-388 calculate offset here in either X or Y...
        System.out.println("OldBounds: " + getX() + ", " + getY() + ", "
                + getWidth() + ", " + getHeight());
        // FIXME, WIRED-388 recalculate the scale factors
        // constrain ratio
        double ratio = (uSize + wSize) / (vSize + wSize);
        if (width > height) {
            super.setBounds(x, y, (int) (ratio * height), height);
        } else {
            super.setBounds(x, y, width, (int) (width / ratio));
        }
    }
}
