package hep.wired.heprep.plugin;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.net.URL;
import java.util.*;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import hep.graphics.heprep.HepRep;
import hep.graphics.heprep.HepRepConversionException;
import hep.graphics.heprep.HepRepConverter;
import hep.graphics.heprep.HepRepProvider;
import hep.graphics.heprep.HepRepType;
import hep.graphics.heprep.HepRepTypeTree;
import hep.graphics.heprep.HepRepViewer;
import hep.graphics.heprep.ref.DefaultHepRep;
import hep.graphics.heprep.util.HepRepUtil;
import hep.wired.heprep.graphicspanel.ControlPanel;
import hep.wired.heprep.graphicspanel.HepRepGraphicsMode;
import hep.wired.heprep.graphicspanel.HepRepPanel;
import hep.wired.heprep.graphicspanel.HepRepPanelModel;
import hep.wired.heprep.image.WiredImage;
import hep.wired.heprep.interaction.CutPanel;
import hep.wired.heprep.interaction.Cuts;
import hep.wired.heprep.interaction.DragRectangleToPick;
import hep.wired.heprep.interaction.DragToMeasure;
import hep.wired.heprep.interaction.HepRepInfoPanel;
import hep.wired.heprep.interaction.HoverToPick;
import hep.wired.heprep.interaction.Interaction;
import hep.wired.heprep.interaction.InteractionPanel;
import hep.wired.heprep.interaction.Picking;
import hep.wired.heprep.projection.CompositeProjection;
import hep.wired.heprep.projection.VariableProjection;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.heprep.tree.TreePanel;
import hep.wired.heprep.util.Keys;
import hep.wired.heprep.util.NullGraphics2D;
import hep.wired.interaction.DefaultInteractionHandler;
import hep.wired.interaction.DragAngledRectangleToScale;
import hep.wired.interaction.DragBallToRotate;
import hep.wired.interaction.DragRectangleToScale;
import hep.wired.interaction.DragToRotate;
import hep.wired.interaction.DragToScale;
import hep.wired.interaction.DragToTranslate;
import hep.wired.plot.WiredPlot;
import hep.wired.plugin.WiredPage;
import hep.wired.plugin.WiredXMLIOFactory;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.InteractionHandler;
import hep.wired.services.RecordPlot;
import hep.wired.util.WiredRegistry;

import org.freehep.application.Application;
import org.freehep.application.ApplicationEvent;
import org.freehep.application.ApplicationListener;
import org.freehep.application.PropertyUtilities;
import org.freehep.application.mdi.ManagedPage;
import org.freehep.application.mdi.PageContext;
import org.freehep.application.mdi.PageEvent;
import org.freehep.application.mdi.PageListener;
import org.freehep.application.studio.Plugin;
import org.freehep.application.studio.Studio;
import org.freehep.graphics2d.VectorGraphics;
import org.freehep.jas.plugin.xmlio.XMLPluginIO;
import org.freehep.record.loop.AbstractLoopListener;
import org.freehep.record.loop.LoopEvent;
import org.freehep.record.loop.RecordLoop;
import org.freehep.record.source.RecordSource;
import org.freehep.util.FreeHEPLookup;
import org.freehep.util.Value;
import org.freehep.xml.io.XMLIO;
import org.freehep.xml.io.XMLIOManager;
import org.freehep.xml.io.XMLIORegistry;
import org.freehep.xml.menus.XMLMenuBuilder;
import org.jdom.Element;
import org.xml.sax.SAXException;

/**
 * Defines the plugin for use in JAS3.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: WiredPlugin.java 14093 2013-01-09 21:57:40Z onoprien $
 */
public class WiredPlugin extends Plugin implements HepRepViewer, XMLPluginIO {

// -- Private parts : ----------------------------------------------------------

  private static WiredPlugin _plugin;
  public static final String KEY_ROOT = "hep.wired.";

  private String _propertiesPrefix;
  private Properties _userProperties;

  // Current items

  private InteractionHandler _interactionHandler;
  private PageContext currentPageContext;
  private PageContext controlContext;
  private HepRep _heprep;

  private static final Icon defaultIcon = WiredImage.getIcon("WiredIcon16.png");
  private Icon wiredIcon;
  private XMLMenuBuilder builder;
  private final WiredPluginCommandHandler pluginHandler = new WiredPluginCommandHandler(this);
  private final WiredGlobalCommandHandler globalHandler = new WiredGlobalCommandHandler();
  private Value viewNo;
  private final PlotHandler plotHandler = new PlotHandler();
  private final PageListener pageHandler = new PageHandler();
  
  private ControlPanel interactionPanel;
  private ControlPanel infoPanel;
  private ControlPanel treePanel;
  private ControlPanel cutPanel;
  
  private HepRepGraphicsMode _graphicsMode; // default graphics mode
  
  private Collection/* PageContext */ eventPlots = new ArrayList();
  private LoopHandler loopHandler = new LoopHandler();
  private RecordLoop loop;
  private Collection/* PageContext */ loopPlots = new ArrayList();

  private ArrayList<HepRepPanelModel> _plotModels = new ArrayList(1);
  private int _lastView = 0;

// -- Access to the singleton : ------------------------------------------------

  public static WiredPlugin getPlugin() {
    return _plugin;
  }

// -- Enumeration of interaction handlers : ------------------------------------
  
  /** Enumeration of {@link InteractionHandler}s used by WIRED. */
  public enum IH {
    DEFAULT(DefaultInteractionHandler.getInstance(), 0),
    ZOOM_DRAG(new DragToScale(), KeyEvent.VK_Z),
    ZOOM_RECTANGLE(new DragRectangleToScale(), KeyEvent.VK_X),
    ZOOM_ANGLE(new DragAngledRectangleToScale(), KeyEvent.VK_A),
    ROTATE_BALL(new DragBallToRotate(), KeyEvent.VK_R),
    ROTATE_FLAT(new DragToRotate(), KeyEvent.VK_F),
    TRANSLATE(new DragToTranslate(), KeyEvent.VK_T),
    MEASURE(new DragToMeasure(), KeyEvent.VK_M),
    PICK_HOVER(new HoverToPick(), KeyEvent.VK_H),
    PICK_RECTANGLE(new DragRectangleToPick(), KeyEvent.VK_P);
    private InteractionHandler _ih;
    private int _keyCode;
    IH(InteractionHandler ih, int keyCode) {
      _ih = ih;
      _keyCode = keyCode;
    }
    /** Returns interaction handler associated with this enum constant. */
    public InteractionHandler get() {return _ih;}
    public int keyCode() {return _keyCode;}
    public ControlPanel control() {
      switch (this) {
        case PICK_HOVER:
        case PICK_RECTANGLE:
          return _plugin.getHepRepInfoPanel();
        default:
          return _plugin.getInteractionPanel();
      }
    }
  }

// -- Construction, initialization and shutdown : ------------------------------

  protected void init() throws SAXException, IOException {
    
    _plugin = this;
    WiredRegistry.add(this);
    
    // Extract properties relevant to Wired
    readProperties();

    // NOTE, should just be done in save/restore but that seems too late.
    addXMLIOFactories(null);

    // initialize Wired-specific menus
    final Studio application = getApplication();
    builder = application.getXMLMenuBuilder();
    URL xml = getClass().getResource("WiredPlugin.menus");
    try {
      builder.build(xml);
    } catch (org.xml.sax.SAXParseException e) {
      System.out.println(e);
      System.out.println("at line: " + e.getLineNumber() + " column: "+ e.getColumnNumber());
      throw e;
    }

    // registed handler for Wired commands
    application.getCommandTargetManager().add(pluginHandler);

    // registed PageListener that will react to Wired plot pages being selected, deselected, or closed
    application.getPageManager().addPageListener(pageHandler);

    // FIXME - empty application listener ?
    application.addApplicationListener(new ApplicationListener() {
      public void initializationComplete(ApplicationEvent arg0) {}
      public void aboutToExit(ApplicationEvent event) {
        shutdown();
      }
    });

    // create default graphics mode and register it as PreferencesTopic
    _graphicsMode = new HepRepGraphicsMode();
    application.getLookup().add(_graphicsMode);

//    JLabel lab = new JLabel(" Info panel ");
//    lab.setAlignmentX(0.5f);
//    application.getStatusBar().add(lab);
  }

  protected void postInit() {
    boolean createNewView = PropertyUtilities.getBoolean(Application.getApplication().getAppProperties(), Keys.heprepPluginNewView, false);
    if (createNewView) pluginHandler.onNewWired4();
  }
  
  protected void shutdown() {
    saveProperties();
    super.shutdown();
  }

  
// -- Actions : ----------------------------------------------------------------

  /** Sets current interaction handler. */
  public void setInteractionHandler(InteractionHandler handler) {
//    System.out.println("WiredPlugin.setInteractionHandler "+ (handler == null ? null : handler.getName()));

    WiredPlot plot = getPlot();
    if ( handler == _interactionHandler || ! (plot == null || plot.supports(handler)) ) return;
    _interactionHandler = handler;

    if (plot != null) plot.setInteractionHandler(_interactionHandler);

    ControlPanel panel = getControlPanel();
    if ((panel != null) && (_interactionHandler != null)) {
      panel.setInteractionHandler(_interactionHandler);
    }

//    for (Iterator contexts = loopPlots.iterator(); contexts.hasNext();) {
//      PageContext context = (PageContext) contexts.next();
//      WiredPage page = (WiredPage) context.getPage();
//      RecordPlot plot = page.getRecordPlot();
//      plot.setInteractionHandler(handler);
//      page.setChanged();
//    }
//
//    for (Iterator contexts = eventPlots.iterator(); contexts.hasNext();) {
//      PageContext context = (PageContext) contexts.next();
//      WiredPage page = (WiredPage) context.getPage();
//      RecordPlot plot = page.getRecordPlot();
//      plot.setInteractionHandler(handler);
//      page.setChanged();
//    }
  }

  /** Sets current control panel. */
  public void setControlPanel(ControlPanel control) {
    getControlPage().setControl(control);
  }

  /**
   * Sets current control panel.
   * If switchInteractionHandler is <tt>true</tt>, also sets the control panel's
   * default interaction handler as the current interaction handler.
   */
  public void setControlPanel(ControlPanel control, boolean switchInteractionHandler) {
    getControlPage().setControl(control);
    if (switchInteractionHandler) {
      InteractionHandler ih = control.getDefaultInteractionHandler();
      if (ih != null) setInteractionHandler(ih);
    }
  }

  
// -- Access to components : ---------------------------------------------------

  /** Returns a list of all plot models that are attached to plots. */
  public List<HepRepPanelModel> getPlotModels() {
    return Collections.unmodifiableList(_plotModels);
  }

  private ControlPage getControlPage() {
    if (controlContext == null) {
      controlContext = getApplication().getControlManager().openPage(new ControlPage(), "WIRED", wiredIcon);
    }
    return (ControlPage) controlContext.getPage();
  }

  public InteractionPanel getInteractionPanel() {
    if (interactionPanel == null) {
      interactionPanel = Interaction.getInstance().getPanel();
    }
    return (InteractionPanel) interactionPanel;
  }

  public TreePanel getTreePanel() {
    if (treePanel == null) {
      treePanel = new TreePanel();
    }
    return (TreePanel) treePanel;
  }

  public HepRepInfoPanel getHepRepInfoPanel() {
    if (infoPanel == null) {
      infoPanel = Picking.getInstance().getPanel();
    }
    return (HepRepInfoPanel) infoPanel;
  }

  public CutPanel getCutPanel() {
    if (cutPanel == null) {
      cutPanel = Cuts.getInstance().getPanel();
    }
    return (CutPanel) cutPanel;
  }

  /** Returns default graphics mode. */
  public HepRepGraphicsMode getGraphicsMode() {
    return _graphicsMode;
  }

  
// -- Access to currently selected items : -------------------------------------

  public PageContext getCurrentPageContext() {
    return currentPageContext;
  }

  /** Returns current control page, null if no control page is available. */
  public ControlPanel getControlPanel() {
    return (controlContext == null) ? null : getControlPage().getControl();
  }

  /** Returns current interaction handler. */
  public InteractionHandler getInteractionHandler() {
    return _interactionHandler;
  }

  /** Returns current WiredPlot, <tt>null</tt> if there is none. */
  public WiredPlot getPlot() {
    try {
      return ((WiredPlot)((WiredPage)currentPageContext.getPage()).getRecordPlot());
    } catch (NullPointerException x) {
      return null;
    } catch (ClassCastException xx) {
      return null;
    }
  }

  /** Returns currently loaded HepRep record. */
  public HepRep getRecord() {
    return _heprep;
  }
  
  
// -- Handling user properties : -----------------------------------------------

  /** Sets prefix to be appended to properties used by this plugin before they are saved into application properties. */
  public void setPropertiesPrefix(String prefix) {
    _propertiesPrefix = prefix;
  }

  /** Returns user properties for this plugin. */
  public Properties getProperties() {
    return _userProperties;
  }

  /** Creates Properties that contains only items relevant to this plugin, with prefix removed. */
  private void readProperties() {
    Properties jas3Properties = getApplication().getUserProperties();
    _propertiesPrefix = jas3Properties.getProperty("jas3.prefix." + this.getClass().getName(), "hep.wired.");
    Set<String> keys = jas3Properties.stringPropertyNames();
    Properties out = new Properties();
    int index = _propertiesPrefix.length();
    for (String key : keys) {
      if (key.startsWith(_propertiesPrefix)) out.setProperty(key.substring(index), jas3Properties.getProperty(key));
    }
    _userProperties = new Properties(out);
  }

  /** 
   * Saves WIRED properties into Jas3 properties, adding prefix to keys and removing 
   * properties that have not been modified since last readProperties().
   */
  private void saveProperties() {
    if (_userProperties == null) return;
    Properties jas3Properties = getApplication().getUserProperties();
    jas3Properties.keySet().removeAll(_userProperties.keySet());
    for (Map.Entry<Object,Object> entry : _userProperties.entrySet()) {
      jas3Properties.setProperty(_propertiesPrefix + entry.getKey(), (String)entry.getValue());
    }
  }


// -- Creating and showing Wired plots : ---------------------------------------

  /**
   * Creates a new WIRED plot.
   *
   * @param name  Name of the created plot.
   *              If <tt>null</tt>, the plot name will be set to "View N" (plot with a new model)
   *              or "<model name>-N" (plot with existing model).
   * @param event  Record to be displayed by the created plot.
   * @param panel  Graphics panel to be used for choosing initial parameters for the new plot.
   * @param clone  If <tt>true</tt>, the newly created plot will share model with the specified panel;
   *               if <tt>false</tt>, a copy of the supplied panel's model will be created.
   *               Ignored if <tt>panel</tt> is <tt>null</tt>.
   */
  public WiredPage createWiredView(String name, HepRep event, HepRepPanel panel, boolean clone) {

    if (event != null) checkHepRep(event);

    HepRepPanel wiredPanel = null;
    HepRepPanelModel model = null;
    if (panel == null) {
      if (name == null) name = "View " + ++_lastView;
      model = new HepRepPanelModel(name);
      wiredPanel = new HepRepPanel(_graphicsMode, model);
//      _plotModels.add(wiredPanel.getModel());
    } else {
      if (clone) {
        model = panel.getModel();
        if (name == null) name = model.getName() +"-"+ model.getNextPanelNumber();
        wiredPanel = panel.clone();
      } else {
        if (name == null) name = "View " + ++_lastView;
        wiredPanel = panel.copy();
        model =  wiredPanel.getModel();
        model.setName(name);
//        _plotModels.add(model);
      }
    }

    WiredPlot wiredPlot = new WiredPlot(name, wiredPanel);
    WiredPage wiredPage = new WiredPage(name, wiredPlot, null);

    wiredPlot.addChangeListener(plotHandler);
//    wiredPlot.setRecord(event);
    wiredPlot.setRecord(event == null ? _heprep : event);

    return wiredPage;
  }

  public void showWired(WiredPage wiredPage, Icon icon, boolean attachToLoop) {
    wiredIcon = (icon == null) ? defaultIcon : icon;

    PageContext context = getApplication().getPageManager().openPage(wiredPage, wiredPage.getName(), wiredIcon);

    if (attachToLoop) {
      loopPlots.add(context);
    } else {
      eventPlots.add(context);
    }

    if (attachToLoop) {
      if (loop == null) {
        loop = (RecordLoop) WiredRegistry.lookup(RecordLoop.class);
        if (loop != null) {
          loop.addLoopListener(loopHandler);
        } else {
          throw new RuntimeException("Could not find record loop");
        }
      }
      if (_heprep == null) {
        try {
          Object record = loop.getRecordSource().getCurrentRecord();
          _heprep = convertRecordToHepRep(record);
          wiredPage.getRecordPlot().setRecord(_heprep);
        } catch (IOException x) {
        } catch (IllegalStateException x) {
        } catch (NullPointerException x) {
        }
      }
    }

    ControlPanel currentControl = getControlPanel();
    if (currentControl == null) currentControl = getInteractionPanel();
    setControlPanel(currentControl);
    setInteractionHandler(currentControl.getInteractionHandler());
  }

  public int getNumberOfPlots() {
    return loopPlots.size() + eventPlots.size();
  }

  // Handles events as a HepRepViewer
  public void display(HepRep event) {
    try {
      String name = null;
      String path = null;
      // FIXME: when HEPREP-13 is implemented
      if (event instanceof DefaultHepRep) {
        name = ((DefaultHepRep) event).getProperty("transient.filename");
        path = ((DefaultHepRep) event).getProperty("transient.path");
      }
      if (name == null) {
        name = "Fixed View";
      }
      if (path == null) {
        path = "Unknown Source";
      }

      checkHepRep(event);

      // FIXME: use path as tooltip, when possible
      showWired(createWiredView(name, event, null, false), null, false);
    } catch (Throwable e) {
      getApplication().error("HepRep invalid", e);
    }
  }


// -- XML save/restore : -------------------------------------------------------

  // FIXME, WIRED-391 duplicate from WiredBase
  private void addXMLIOFactories(XMLIORegistry registry) {
    addXMLIOFactory(registry, GraphicsMode.class);
    addXMLIOFactory(registry, HepRepPanel.class);

    addXMLIOFactoryForService(registry, Projection.class);
  }

  // FIXME, WIRED-391 duplicate from WiredBase
  private void addXMLIOFactory(XMLIORegistry registry, Class cls) {
    if ((registry == null) || (registry.getXMLIOFactory(cls) == null)) {
      WiredRegistry.add(new WiredXMLIOFactory(cls));
    }
  }

  // FIXME, WIRED-391 duplicate from WiredBase
  private void addXMLIOFactoryForService(XMLIORegistry registry, Class service) {
    Set allClasses = WiredRegistry.allClasses(service);
    for (Iterator i = allClasses.iterator(); i.hasNext();) {
      addXMLIOFactory(registry, (Class) i.next());
    }
  }

  public void save(XMLIOManager manager, org.jdom.Element el) {
    addXMLIOFactories(manager.getXMLIORegistry());
    for (Iterator i = loopPlots.iterator(); i.hasNext();) {
      PageContext pageContext = (PageContext) i.next();
      Component page = pageContext.getPage();
      if (page instanceof XMLIO) {
        Element pageNode = manager.save((XMLIO) page);
        pageNode.setAttribute("attachToLoop", Boolean.toString(true));
        el.addContent(pageNode);
      }
    }
    for (Iterator i = eventPlots.iterator(); i.hasNext();) {
      PageContext pageContext = (PageContext) i.next();
      Component page = pageContext.getPage();
      if (page instanceof XMLIO) {
        el.addContent(manager.save((XMLIO) page));
      }
    }
  }

  public int restore(int level, XMLIOManager manager, org.jdom.Element el) {
    // System.out.println("Restoring WiredPlugin at "+level);
    if (level == RESTORE_PLOTS_IN_PAGES) {
      for (Iterator i = el.getChildren().iterator(); i.hasNext();) {
        Element pageNode = (Element) i.next();
        WiredPage page = (WiredPage) manager.restore(pageNode);
        boolean attachToLoop = Boolean.getBoolean(pageNode.getAttributeValue("attachToLoop", "true"));
        showWired(page, null, attachToLoop);
        // rename page
        // page.setName("View "+viewNo.getInt());
        viewNo.set(viewNo.getInt() + 1);
      }

      return RESTORE_DONE;
    } else {
      addXMLIOFactories(manager.getXMLIORegistry());
      return RESTORE_PLOTS_IN_PAGES;
    }
  }

  public List getVariables() {
    List variables = new ArrayList();

    if (currentPageContext == null) {
      return variables;
    }

    Component page = currentPageContext.getPage();
    if (page instanceof WiredPage) {
      WiredPage wiredPage = (WiredPage) page;
      RecordPlot plot = wiredPage.getRecordPlot();
      GraphicsPanel panel = plot.getGraphicsPanel();
      if (panel instanceof HepRepPanel) {
        HepRepPanel heprepPanel = (HepRepPanel) panel;

//        // graphicsmode variables
//        HepRepGraphicsMode mode = (HepRepGraphicsMode) heprepPanel.getGraphicsMode();
//        variables.addAll(mode.getVariables());

        // projection variables
        variables.addAll(getVariables(heprepPanel.getProjection()));
      }
    }

    return variables;
  }

  private List getVariables(Projection projection) {
    List variables = new ArrayList();
    if (projection instanceof VariableProjection) {
      variables.addAll(((VariableProjection) projection).getVariables());
    } else if (projection instanceof CompositeProjection) {
      CompositeProjection composite = (CompositeProjection) projection;
      for (Iterator i = composite.getProjections().iterator(); i.hasNext();) {
        variables.addAll(getVariables((Projection) i.next()));
      }
    }
    return variables;
  }


// -- Converting records to HepRep and validating : ----------------------------

  /**
   * Converts the supplied record object to <tt>HepRep</tt>.
   * @throws HepRepConversionException if conversion is impossible.
   */
  private HepRep convertRecordToHepRep(Object record) {
    HepRep heprep = null;
    if (record != null) {
      FreeHEPLookup registry = ((Studio) Application.getApplication()).getLookup();
      HepRepProvider converter = HepRepUtil.getHepRepProvider(registry, record);
      if (converter == null) { // for backward compatibility
        HepRepConverter oldConverter = HepRepUtil.getHepRepConverter(registry, record.getClass());
        if (oldConverter == null) {
          throw new HepRepConversionException();
        } else {
          heprep = oldConverter.convert(record);
        }
      } else {
        heprep = converter.convert(record);
      }
      checkHepRep(heprep);  // patch and check heprep for validity. FIXME: look at Mark's code - what if invalid ? diagnostics ?
    }
    return heprep;
  }

  private HepRepPanel checkHepRepPanel;
  private VectorGraphics checkHepRepGraphics;
  public void checkHepRep(HepRep heprep) {
    // patch the HepRep for a bug with DrawAs values in GEANT4-xxx (only in Geant4.7.1)
    patchG471bug(heprep);
    // check if the heprep is correct
    if (checkHepRepPanel == null) {
      checkHepRepGraphics = new NullGraphics2D();
      checkHepRepPanel = new HepRepPanel(_graphicsMode);
    }
    checkHepRepPanel.setRecord(heprep);
    checkHepRepPanel.draw(checkHepRepGraphics, true, null, null, false);
  }

  /** Patch the HepRep for a bug with DrawAs values in GEANT4-60 (only in Geant4.7.1) */
  private void patchG471bug(HepRep heprep) {
    if (heprep == null) return;
    for (Object typeTree : heprep.getTypeTreeList()) {
      patchG471bug( ((HepRepTypeTree)typeTree).getTypeList() );
    }
  }

  /** Patch the HepRep for a bug with DrawAs values in GEANT4-60 (only in Geant4.7.1) */
  private void patchG471bug(Collection/* <HepRepType> */ typeList) {
    for (Iterator i = typeList.iterator(); i.hasNext();) {
      HepRepType type = (HepRepType) i.next();
      if (type.getName().equals("*Face")) {
        type.addAttValue("DrawAs", "Polygon");
      }
      patchG471bug(type.getTypeList());
    }
  }


// -- Plot and Page handler classes : ------------------------------------------

  class PlotHandler implements ChangeListener {

    public void stateChanged(ChangeEvent event) {
      if (controlContext != null) {
        controlContext.requestShow();
      }
    }
  }

  /**
   * Listens to wired plot pages close/select/deselect events.
   * When the page is selected or deselected, setSelection() is called on both old and new
   * plot from WiredPage before this PageHandler gets notified.
   */
  class PageHandler implements PageListener {

    public void pageChanged(PageEvent event) {
      
      if (!(event.getPageContext().getPage() instanceof WiredPage)) return;  // ignore non wired pages.
      RecordPlot plot = ((WiredPage) event.getPageContext().getPage()).getRecordPlot();

      switch (event.getID()) {
        case PageEvent.PAGESELECTED: {
          currentPageContext = event.getPageContext();
          getApplication().getCommandTargetManager().add(globalHandler);
          if (plot instanceof WiredPlot) {
            WiredPlot wiredPlot = (WiredPlot) plot;
            ControlPanel control = getControlPanel();
            if (control != null) control.setSelected(true);
            // make sure interaction handler is valid
            if (!wiredPlot.supports(getInteractionHandler())) setInteractionHandler(null);
          }
          if (controlContext != null) controlContext.requestShow();
          break;
        }
        case PageEvent.PAGEDESELECTED: {
          getApplication().getCommandTargetManager().remove(globalHandler);
          if (plot instanceof WiredPlot) {
            ControlPanel control = getControlPanel();
            if (control != null) control.setSelected(false);
          }
          currentPageContext = null;
          break;
        }
        case PageEvent.PAGECLOSED: {
          // plot.destroy();
          if (plot instanceof WiredPlot) {
            ControlPanel control = getControlPanel();
            if (control != null) control.setSelected(false);
          }
          PageContext context = event.getPageContext();
          eventPlots.remove(context);
          loopPlots.remove(context);
//          plot.setRecord(null);
          plot.setInteractionHandler(null);
          plot.removeChangeListener(plotHandler);
          currentPageContext = null;

          // close all wired info
          if (getNumberOfPlots() == 0) {
            if (controlContext != null) {
              controlContext.close();
              controlContext = null;
            }
            treePanel = null;

            if (loop != null) {
              loop.removeLoopListener(loopHandler);
              loop = null;
            }
          }
          break;
        }
        default:
          break;
      }

      pluginHandler.setChanged();
      globalHandler.setChanged();
    }
  }

// -- LoopHandler class and HepRep fixing: -------------------------------------

  /** Handles events coming from a record loop. All loopPlots will be updated. */
  class LoopHandler extends AbstractLoopListener {
    
    protected void reset(LoopEvent event) {
      setRecord(null);
    }

    protected void suspend(LoopEvent event) {
      RecordLoop loop = (RecordLoop) event.getSource();
      RecordSource source = loop.getRecordSource();
      if (source != null) {
        Object record = null;
        try {
          record = source.getCurrentRecord();
        } catch (IOException e) {
          return;  // ignore
        } catch (Exception x) {
          getApplication().error("WIRED failed to get record from the source", x);
        }
        try {
          setRecord(record);
        } catch (Throwable e) {
          getApplication().error("HepRep invalid", e);
        }
      }
    }

    void setRecord(Object record) {
      HepRep heprep = null;
      try {
        heprep = convertRecordToHepRep(record);
      } catch (HepRepConversionException x) {
      }
      setRecord(heprep);
    }

    void setRecord(HepRep heprep) {
      _heprep = heprep;
//      if (heprep == null) return;  // FIXME - need to display blanc plot

      // Check/fix heprep
      if (heprep != null) {
        checkHepRep(heprep);
        patchG471bug(heprep);  // patch the HepRep for a bug with DrawAs values in GEANT4-60 (only in Geant4.7.1)
      }

      // Tell all plots
      Iterator contexts = loopPlots.iterator();
      while (contexts.hasNext()) {
        PageContext context = (PageContext) contexts.next();
        WiredPage page = (WiredPage) context.getPage();
        RecordPlot plot = page.getRecordPlot();
        plot.setRecord(heprep); // tell the plot, it's up to the plot to notify its model
        page.setChanged();
      }

      // Tell all ControlPanels
      getControlPage().setRecord(heprep);

      // Re-select current control panel
      ControlPanel control = getControlPanel();
      if (control != null && getPlot() != null) control.setSelected(true);
    }
  }


// -- ControlPage class : ------------------------------------------------------

  /**
   * Manager class for all ControlPanels.
   * Control Panel to disable close button if WIRED views exist.
   */
  final class ControlPage extends JPanel implements ManagedPage {

    private final JTabbedPane handler;
    private ControlPanel control;

    private class Tab extends JLabel {
      int index;
      MouseListener listener;
      Tab(int i, ControlPanel c) {
        index = i;
        addMouseListener(new MouseAdapter() {
          public void mousePressed(MouseEvent e) {
            if (index == handler.getSelectedIndex()) {
              setInteractionHandler(((ControlPanel)handler.getComponentAt(index)).getDefaultInteractionHandler());
            }
            handler.setSelectedIndex(index);
          }
        });
        setIcon(c.getIcon());
      }
      public String getText() {return handler.getTitleAt(index);}
    }

    ControlPage() {
      setLayout(new BorderLayout());
      handler = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
      add(getInteractionPanel());
      add(getHepRepInfoPanel());
//      add(getVariablePanel());
      add(getCutPanel());
      handler.addChangeListener(new ChangeListener() {
        public void stateChanged(ChangeEvent event) {
          setControl((ControlPanel) handler.getSelectedComponent());
        }
      });
      add(handler, BorderLayout.CENTER);
    }

    void add(ControlPanel control) {
      handler.addTab(control.getName(), control.getIcon(), control, control.getToolTipText());
      int index = handler.indexOfComponent(control);
      handler.setTabComponentAt(index, new Tab(index,control));
    }

    ControlPanel getControl() {
      return control;
    }

    /**
     * Switch from one selected ControlPage to another, calling setSelection(...) on both
     * and setChanged() on plugin handler.
     */
    void setControl(ControlPanel control) {
      if (this.control != null) this.control.setSelected(false);
      this.control = control;
      if (control != null) {
        control.setSelected(true);
        handler.setSelectedComponent(control);
      }
      pluginHandler.setChanged();
    }

    /** Sets record to all ControlPages. */
    void setRecord(HepRep heprep) {
      int n = handler.getTabCount();
      for (int i=0; i<n; i++) {
        ((ControlPanel)( handler.getComponentAt(i))).setRecord(heprep);
      }
    }

    // ManagedPage interface
 
    public boolean close() {
      return getNumberOfPlots() == 0;
    }

    public void setPageContext(PageContext context) {}

    public void pageSelected() {}

    public void pageDeselected() {}

    public void pageIconized() {}

    public void pageDeiconized() {}

    public void pageClosed() {
      controlContext = null;
    }
  }
}
