package hep.wired.heprep.plugin;

import hep.wired.heprep.graphicspanel.ControlPanel;
import hep.wired.heprep.graphicspanel.HepRepPanel;
import hep.wired.heprep.graphicspanel.HepRepPanelModel;
import hep.wired.plot.WiredPlot;
import java.net.MalformedURLException;
import java.net.URL;

import org.freehep.application.Application;
import org.freehep.application.studio.Studio;
import org.freehep.jas.services.WebBrowser;
import org.freehep.util.Value;
import org.freehep.util.commanddispatcher.BooleanCommandState;
import org.freehep.util.commanddispatcher.CommandProcessor;
import org.freehep.util.commanddispatcher.CommandState;

/**
 * Defines a command handler for plugin commands.
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredPluginCommandHandler.java 13931 2012-02-09 00:58:03Z onoprien $
 */
public class WiredPluginCommandHandler extends CommandProcessor {
  
// -- Private parts : ----------------------------------------------------------

//  private Value viewNo;
  private WiredPlugin plugin;
  private Studio studio;

// -- Construction : -----------------------------------------------------------

  public WiredPluginCommandHandler(WiredPlugin plugin) {
    this.plugin = plugin;
    studio = (Studio) Application.getApplication();
  }

// -- New plot : ---------------------------------------------------------------

  public void onNewWired4() {
    plugin.showWired(plugin.createWiredView(null, null, null, false), null, true);
  }

  public void onCloneWired4() {
    HepRepPanel panel = null;
    try {
      panel = (HepRepPanel) plugin.getPlot().getGraphicsPanel();
    } catch (ClassCastException x) {
    } catch (NullPointerException xx) {}
    plugin.showWired(plugin.createWiredView(null, null, panel, true), null, false);
//    viewNo.set(viewNo.getInt() + 1);
  }

  public void onCopyWired4() {
    HepRepPanel panel = null;
    try {
      panel = (HepRepPanel) plugin.getPlot().getGraphicsPanel();
    } catch (ClassCastException x) {
    } catch (NullPointerException xx) {}
    plugin.showWired(plugin.createWiredView(null, null, panel, false), null, false);
//    viewNo.set(viewNo.getInt() + 1);
  }

// -- Selecting Control Panels : -----------------------------------------------

  public void enableInteraction(BooleanCommandState state) {
    state.setEnabled(plugin.getCurrentPageContext() != null);
    state.setSelected(plugin.getInteractionPanel().equals(plugin.getControlPanel()));
    setChanged();
  }

  public void onInteraction(boolean on) {
    if (on) {
      ControlPanel cp = plugin.getInteractionPanel();
      if (cp == plugin.getControlPanel()) {
        plugin.setInteractionHandler(cp.getDefaultInteractionHandler());
        setChanged();
      } else {
        plugin.setControlPanel(cp);
      }
    }
  }

  public void enablePicking(BooleanCommandState state) {
    state.setEnabled(plugin.getCurrentPageContext() != null);
    state.setSelected(plugin.getHepRepInfoPanel().equals(plugin.getControlPanel()));
    setChanged();
  }

  public void onPicking(boolean on) {
    if (on) {
      ControlPanel cp = plugin.getHepRepInfoPanel();
      if (cp == plugin.getControlPanel()) {
        plugin.setInteractionHandler(cp.getDefaultInteractionHandler());
        setChanged();
      } else {
        plugin.setControlPanel(cp);
      }
    }
  }

  public void enableTreeSelection(BooleanCommandState state) {
    state.setEnabled(plugin.getCurrentPageContext() != null);
    state.setSelected(plugin.getTreePanel().equals(plugin.getControlPanel()));
    setChanged();
  }

  public void onTreeSelection(boolean on) {
    if (on) plugin.setControlPanel(plugin.getTreePanel());
  }

//  public void enableVariables(BooleanCommandState state) {
//    state.setEnabled(plugin.getCurrentPageContext() != null);
//    state.setSelected(plugin.getVariablePanel().equals(plugin.getControlPanel()));
//    setChanged();
//  }
//
//  public void onVariables(boolean on) {
//    if (on) plugin.setControlPanel(plugin.getVariablePanel());
//  }

  public void enableCuts(BooleanCommandState state) {
    state.setEnabled(plugin.getCurrentPageContext() != null);
    state.setSelected(plugin.getCutPanel().equals(plugin.getControlPanel()));
    setChanged();
  }

  public void onCuts(boolean on) {
    if (on) plugin.setControlPanel(plugin.getCutPanel());
  }

// -- Miscellaneous : ----------------------------------------------------------

  public void enableWiredHome(CommandState state) {
    hasWebBrowser(state);
  }

  public void onWiredHome() {
    try {
// FIXME, should not happen here ?
      studio.getUserProperties().setProperty("hep.wired", "http://wired4.freehep.org/");
      showWebPage("hep.wired");
    } catch (MalformedURLException e) {
      studio.error("Invalid URL", e);
    }
  }

  public void enableWiredUserManual(CommandState state) {
    hasWebBrowser(state);
  }

  public void onWiredUserManual() {
    try {
// FIXME, should not happen here ?
      studio.getUserProperties().setProperty("hep.wired.manual", "http://wired4.freehep.org/manual/index.html");
      showWebPage("hep.wired.manual");
    } catch (MalformedURLException e) {
      studio.error("Invalid URL", e);
    }
  }

  private void hasWebBrowser(CommandState state) {
    state.setEnabled(studio.getLookup().lookup(WebBrowser.class) != null);
  }

  private void showWebPage(String property) throws MalformedURLException {
    WebBrowser webBrowser = ((WebBrowser) studio.getLookup().lookup(WebBrowser.class));
    webBrowser.showURL(new URL(studio.getUserProperties().getProperty(property)), true);
  }
}
