// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.services.ViewPort;

import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws a cubic bezier curve specified by the HepRepPoint control points.
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsCurve.java 8598 2006-08-14 20:38:02Z duns $
 */

public class DrawAsCurve extends AbstractDrawAs {

    // FIXME WIRED-470, hardcoded
    private double uvw[][] = new double[3][3000];

    public String getKey() {
        return "curve";
    }

    public String getName() {
        return "Curve";
    }

    public String getDescription() {
        return "Draws cubic bezier curves.";
    }

    private double[][] createCurve(double[][] uvw, int n) {
        // first point is already correct
        for (int i = 1; i < n; i += 3) {
            double t = (i % 3.0) / 3.0;
            double f0 = Math.pow(1 - t, 3);
            double f1 = 3 * t * Math.pow(1 - t, 2);
            double f2 = 3 * Math.pow(t, 2) * (1 - t);
            double f3 = Math.pow(t, 3);
            uvw[X][i] = f0 * uvw[X][i - 1] + f1 * uvw[X][i] + f2
                    * uvw[X][i + 1] + f3 * uvw[X][i + 2];
            uvw[Y][i] = f0 * uvw[Y][i - 1] + f1 * uvw[Y][i] + f2
                    * uvw[Y][i + 1] + f3 * uvw[Y][i + 2];
            uvw[Z][i] = f0 * uvw[Z][i - 1] + f1 * uvw[Z][i] + f2
                    * uvw[Z][i + 1] + f3 * uvw[Z][i + 2];
        }
        return uvw;
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        // FIXME strictly not correct, since we need to skip every 3 points
        int n = fillArray(uvw, instance);
        if (n == 0)
            return;

        uvw = createCurve(uvw, n);
        uvw = projection.transform(viewPort, uvw, n);
        graphics.drawPolyline(uvw[U], uvw[V], n);
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        int n = fillArray(uvw, instance);
        if (n == 0)
            return;

        uvw = createCurve(uvw, n);
        uvw = projection.transform(viewPort, uvw, n);

        graphics.setColor(atts.getFrameColor());
        graphics.setLineWidth(atts.getFrameWidth() * 2 + atts.getLineWidth());
        graphics.drawPolyline(uvw[U], uvw[V], n);
        graphics.setLineWidth(atts.getLineWidth());
        graphics.setColor(atts.getColor());
    }
}
