// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.HepRepPoint;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.heprep.util.PolyPoint;
import hep.wired.services.ViewPort;
import hep.wired.util.Matrix3D;

import java.awt.Color;
import java.util.Iterator;

import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws one or more ellipsoids, centered on HepRepPoints with given "Radius",
 * "Radius2" and "Radius3", positioned by "Phi", "Theta" and "Omega".
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsEllipsoid.java 8598 2006-08-14 20:38:02Z duns $
 */

public class DrawAsEllipsoid extends AbstractDrawAs {

    private double uvw[][][] = new double[3][3][PolyPoint.ELLIPSEPOINTS];

    public String getKey() {
        return "ellipsoid";
    }

    public String getName() {
        return "Ellipsoid";
    }

    public String getDescription() {
        return "Draws a HepRepInstance as an Ellipse.";
    }

    private double[][][] createEllipsoids(double r1, double r2, double r3,
            double x, double y, double z, double phi, double theta,
            double omega, double[][][] uvw) {
        PolyPoint.ellipsoid(r1, r2, r3, x, y, z, phi, theta, omega, uvw);
        return uvw;
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        Iterator iterator = instance.getPoints().iterator();
        while (iterator.hasNext()) {
            HepRepPoint center = (HepRepPoint) iterator.next();

            uvw = createEllipsoids(center.getAttValue("radius").getDouble(),
                    center.getAttValue("radius2").getDouble(), center
                            .getAttValue("radius3").getDouble(), center.getX(),
                    center.getY(), center.getZ(), center.getAttValue("phi")
                            .getDouble(), center.getAttValue("theta")
                            .getDouble(), center.getAttValue("omega")
                            .getDouble(), uvw);
            uvw[X] = projection.transform(viewPort, uvw[X],
                    PolyPoint.ELLIPSEPOINTS);
            uvw[Y] = projection.transform(viewPort, uvw[Y],
                    PolyPoint.ELLIPSEPOINTS);
            uvw[Z] = projection.transform(viewPort, uvw[Z],
                    PolyPoint.ELLIPSEPOINTS);

            if (atts.isFilled()) {
                graphics.setColor(atts.getFillColor());
                graphics.fillPolygon(uvw[X][U], uvw[X][V],
                        PolyPoint.ELLIPSEPOINTS);
                graphics.fillPolygon(uvw[Y][U], uvw[Y][V],
                        PolyPoint.ELLIPSEPOINTS);
                graphics.fillPolygon(uvw[Z][U], uvw[Z][V],
                        PolyPoint.ELLIPSEPOINTS);
                graphics.setColor(atts.getColor());
            }
            graphics.drawPolygon(uvw[X][U], uvw[X][V], PolyPoint.ELLIPSEPOINTS);
            graphics.drawPolygon(uvw[Y][U], uvw[Y][V], PolyPoint.ELLIPSEPOINTS);
            graphics.drawPolygon(uvw[Z][U], uvw[Z][V], PolyPoint.ELLIPSEPOINTS);
        }
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        graphics.setColor(atts.getFrameColor());
        graphics.setLineWidth(atts.getFrameWidth() * 2 + atts.getLineWidth());

        Iterator iterator = instance.getPoints().iterator();
        while (iterator.hasNext()) {
            HepRepPoint center = (HepRepPoint) iterator.next();
            uvw = createEllipsoids(center.getAttValue("radius").getDouble(),
                    center.getAttValue("radius2").getDouble(), center
                            .getAttValue("radius3").getDouble(), center.getX(),
                    center.getY(), center.getZ(), center.getAttValue("phi")
                            .getDouble(), center.getAttValue("theta")
                            .getDouble(), center.getAttValue("omega")
                            .getDouble(), uvw);
            uvw[X] = projection.transform(viewPort, uvw[X],
                    PolyPoint.ELLIPSEPOINTS);
            uvw[Y] = projection.transform(viewPort, uvw[Y],
                    PolyPoint.ELLIPSEPOINTS);
            uvw[Z] = projection.transform(viewPort, uvw[Z],
                    PolyPoint.ELLIPSEPOINTS);

            graphics.drawPolygon(uvw[X][U], uvw[X][V], PolyPoint.ELLIPSEPOINTS);
            graphics.drawPolygon(uvw[Y][U], uvw[Y][V], PolyPoint.ELLIPSEPOINTS);
            graphics.drawPolygon(uvw[Z][U], uvw[Z][V], PolyPoint.ELLIPSEPOINTS);
        }

        graphics.setLineWidth(atts.getLineWidth());
        graphics.setColor(atts.getColor());
    }
}
