// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.heprep.util.Arrow;
import hep.wired.services.ViewPort;

import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws one or more line segments, given by HepRepPoints.
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsLine.java 8598 2006-08-14 20:38:02Z duns $
 */

public class DrawAsLine extends AbstractDrawAs {

    // FIXME WIRED-470, limit hardcoded
    private double uvw[][] = new double[3][3000];

    public String getKey() {
        return "line";
    }

    public String getName() {
        return "Line";
    }

    public String getDescription() {
        return "Draws line segments from point 0 till point n.";
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        int n = fillArray(uvw, instance);
        if (n == 0)
            return;

        uvw = projection.transform(viewPort, uvw, n);
        graphics.drawPolyline(uvw[U], uvw[V], n);

        // NOTE: arrows are redrawing last part of line
        if ((atts.hasLineArrow() == Attributes.ARROW_START)
                || (atts.hasLineArrow() == Attributes.ARROW_BOTH)) {
            graphics.fill(new Arrow(uvw[U][1], uvw[V][1], uvw[U][0], uvw[V][0],
                    20, 16, 70, false));
        }

        // NOTE: arrows are redrawing last part of line
        if ((atts.hasLineArrow() == Attributes.ARROW_END)
                || (atts.hasLineArrow() == Attributes.ARROW_BOTH)) {
            graphics.fill(new Arrow(uvw[U][n - 2], uvw[V][n - 2],
                    uvw[U][n - 1], uvw[V][n - 1], 20, 16, 70, false));
        }
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        int n = fillArray(uvw, instance);
        if (n == 0)
            return;

        uvw = projection.transform(viewPort, uvw, n);

        graphics.setColor(atts.getFrameColor());
        graphics.setLineWidth(atts.getFrameWidth() * 2 + atts.getLineWidth());
        graphics.drawPolyline(uvw[U], uvw[V], n);
        graphics.setLineWidth(atts.getLineWidth());
        graphics.setColor(atts.getColor());

        // FIXME WIRED-392 arrow frame missing
    }
}
