// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.HepRepPoint;
import hep.wired.heprep.graphicspanel.HepRepGraphicsMode;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.services.ViewPort;

import java.util.Iterator;

import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws one or more marker of type "MarkName" and size "MarkSize", centered at
 * HepRepPoints.
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsPoint.java 13945 2012-02-27 00:27:14Z onoprien $
 */

public class DrawAsPoint extends AbstractDrawAs {

    private double[] uvw = new double[3];

    public String getKey() {
        return "point";
    }

    public String getName() {
        return "Point";
    }

    public String getDescription() {
        return "Draws any HepRepPoints as a Marker (square, circle, ...) symbol.";
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        HepRepGraphicsMode heprepMode = (HepRepGraphicsMode) mode;
        for (Iterator i = instance.getPoints().iterator(); i.hasNext();) {

            HepRepPoint p = (HepRepPoint) i.next();
            uvw[U] = p.getX();
            uvw[V] = p.getY();
            uvw[W] = p.getZ();

            uvw = projection.transform(viewPort, uvw);

            if (atts.isFilled() && heprepMode.fillBoxes) {
                graphics.fillAndDrawSymbol(uvw[U], uvw[V], atts.getMarkSize()
                        * heprepMode.markSizeMultiplier, atts.getMarkSymbol(),
                        atts.getFillColor());
            } else {
                graphics.drawSymbol(uvw[U], uvw[V], atts.getMarkSize()
                        * heprepMode.markSizeMultiplier,
                        heprepMode.useLineWidth ? atts.getMarkSymbol()
                                : VectorGraphics.SYMBOL_BOX);
            }
        }
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        HepRepGraphicsMode heprepMode = (HepRepGraphicsMode) mode;
        graphics.setColor(atts.getFrameColor());
        for (Iterator i = instance.getPoints().iterator(); i.hasNext();) {

            HepRepPoint p = (HepRepPoint) i.next();
            uvw[U] = p.getX();
            uvw[V] = p.getY();
            uvw[W] = p.getZ();

            uvw = projection.transform(viewPort, uvw);

            graphics.fillSymbol(uvw[U], uvw[V], atts.getFrameWidth() * 2
                    + atts.getMarkSize() * heprepMode.markSizeMultiplier, atts
                    .getMarkSymbol());
        }
        graphics.setColor(atts.getColor());
    }
}
