// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.services.ViewPort;

import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws a closed set of lines, given a set of HepRepPoints.
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsPolygon.java 8598 2006-08-14 20:38:02Z duns $
 */

public class DrawAsPolygon extends AbstractDrawAs {

    // FIXME WIRED-470, limited.
    private double uvw[][] = new double[3][3000];

    public String getKey() {
        return "polygon";
    }

    public String getName() {
        return "Polygon";
    }

    public String getDescription() {
        return "Draws a closed set of line segments for all HepRepPoints.";
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        int n = fillArray(uvw, instance);
        if (n == 0)
            return;

        uvw = projection.transform(viewPort, uvw, n);

        if (atts.isFilled()) {
            graphics.setColor(atts.getFillColor());
            graphics.fillPolygon(uvw[U], uvw[V], n);
            graphics.setColor(atts.getColor());
        }
        graphics.drawPolygon(uvw[U], uvw[V], n);
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        int n = fillArray(uvw, instance);
        if (n == 0)
            return;

        uvw = projection.transform(viewPort, uvw, n);

        graphics.setColor(atts.getFrameColor());
        graphics.setLineWidth(atts.getFrameWidth() * 2 + atts.getLineWidth());
        graphics.drawPolygon(uvw[U], uvw[V], n);
        graphics.setLineWidth(atts.getLineWidth());
        graphics.setColor(atts.getColor());
    }
}
