// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.services.ViewPort;

import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws a prism, where one face is given by the first half of the HepRepPoints
 * and the other by the second half of the HepRepPoints.
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsPrism.java 8598 2006-08-14 20:38:02Z duns $
 */

public class DrawAsPrism extends AbstractDrawAs {

    // FIXME WIRED-470, hardcoded (this limit needs to be even!)
    // NOTE inner needs to at first contain all the points.
    private double inner[][] = new double[3][100];

    private double outer[][] = new double[3][100];

    private double side[][][] = new double[100][3][4];

    public String getKey() {
        return "prism";
    }

    public String getName() {
        return "Prism";
    }

    public String getDescription() {
        return "Draws a closed set of line segments forming a Prism.";
    }

    private int createPrism(int n) {
        int k = n / 2;

        // outer
        for (int i = k; i < n; i++) {
            outer[U][i - k] = inner[U][i];
            outer[V][i - k] = inner[V][i];
            outer[W][i - k] = inner[W][i];
        }
        // sides of prism - make each a polygon.
        for (int i = 0; i < k; i++) {
            side[i][U][0] = outer[U][i];
            side[i][V][0] = outer[V][i];
            side[i][W][0] = outer[W][i];

            side[i][U][1] = inner[U][i];
            side[i][V][1] = inner[V][i];
            side[i][W][1] = inner[W][i];

            int next = (i + 1 == k) ? 0 : i + 1;
            side[i][U][2] = inner[U][next];
            side[i][V][2] = inner[V][next];
            side[i][W][2] = inner[W][next];

            side[i][U][3] = outer[U][next];
            side[i][V][3] = outer[V][next];
            side[i][W][3] = outer[W][next];
        }

        return k;
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        int n = fillArray(inner, instance);
        if (n == 0)
            return;

        int k = createPrism(n);

        inner = projection.transform(viewPort, inner, k);
        outer = projection.transform(viewPort, outer, k);
        for (int i = 0; i < k; i++) {
            side[i] = projection.transform(viewPort, side[i], 4);
        }

        if (atts.isFilled()) {
            graphics.setColor(atts.getFillColor());
            graphics.fillPolygon(inner[U], inner[V], k);
            graphics.fillPolygon(outer[U], outer[V], k);
            for (int i = 0; i < k; i++) {
                graphics.fillPolygon(side[i][U], side[i][V], 4);
            }
            graphics.setColor(atts.getColor());
        }
        graphics.drawPolygon(inner[U], inner[V], k);
        graphics.drawPolygon(outer[U], outer[V], k);
        for (int i = 0; i < k; i++) {
            graphics.drawPolygon(side[i][U], side[i][V], 4);
        }
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        int n = fillArray(inner, instance);
        if (n == 0)
            return;

        int k = createPrism(n);

        inner = projection.transform(viewPort, inner, k);
        outer = projection.transform(viewPort, outer, k);
        for (int i = 0; i < k; i++) {
            side[i] = projection.transform(viewPort, side[i], 4);
            side[i] = viewPort.transform(side[i], 4);
        }

        graphics.setColor(atts.getFrameColor());
        graphics.setLineWidth(atts.getFrameWidth() * 2 + atts.getLineWidth());
        graphics.drawPolygon(inner[U], inner[V], k);
        graphics.drawPolygon(outer[U], outer[V], k);
        for (int i = 0; i < k; i++) {
            graphics.drawPolygon(side[i][U], side[i][V], 4);
        }
        graphics.setLineWidth(atts.getLineWidth());
        graphics.setColor(atts.getColor());
    }
}
