// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.util.HepRepFont;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.services.ViewPort;
import hep.wired.viewport.RectangularViewPort;

import java.awt.Color;
import java.awt.Font;

import org.freehep.graphics2d.TagString;
import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws text in the plot, positioned using "HPos" and "VPos" and aligned using
 * "HAlign" and "VAlign".
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsText.java 8598 2006-08-14 20:38:02Z duns $
 */

public class DrawAsText extends AbstractDrawAs {

    public String getKey() {
        return "text";
    }

    public String getName() {
        return "Text";
    }

    public String getDescription() {
        return "Draws Text.";
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        Font font = graphics.getFont();
        Color color = graphics.getColor();
        graphics.setFont(new Font(instance.getAttValue("fontName").getString(),
                HepRepFont.getStyle(instance.getAttValue("fontStyle")
                        .getString()), instance.getAttValue("fontSize")
                        .getInteger()));
        graphics.setColor(instance.getAttValue("fontColor").getColor());
        TagString string = new TagString(instance.getAttValue("text")
                .getString());

        if (viewPort instanceof RectangularViewPort) {
            RectangularViewPort rvp = (RectangularViewPort) viewPort;

            graphics.drawString(string, instance.getAttValue("hPos")
                    .getDouble()
                    * rvp.getComponentWidth(), instance.getAttValue("vPos")
                    .getDouble()
                    * rvp.getComponentHeight(), VectorGraphics
                    .getTextAlignment(instance.getAttValue("hAlign")
                            .getString()), VectorGraphics
                    .getTextAlignment(instance.getAttValue("vAlign")
                            .getString()), instance.getAttValue("fontHasFrame")
                    .getBoolean(), instance.getAttValue("fontFrameColor")
                    .getColor(), instance.getAttValue("fontFrameWidth")
                    .getDouble(), instance.getAttValue("fontHasBanner")
                    .getBoolean(), instance.getAttValue("fontBannerColor")
                    .getColor());
        }

        graphics.setColor(color);
        graphics.setFont(font);
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        // Framing done in draw
    }
}
