package hep.wired.heprep.tree;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import hep.graphics.heprep.HepRepAttValue;
import hep.graphics.heprep.HepRepInstance;
import hep.wired.heprep.graphicspanel.ControlPanel;
import hep.wired.heprep.graphicspanel.HepRepPanel;
import hep.wired.heprep.graphicspanel.HepRepPanelModel;
import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.image.WiredBaseImage;
import hep.wired.plot.WiredPlot;
import hep.wired.plugin.WiredPage;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;
import org.freehep.application.mdi.PageContext;
import org.freehep.swing.layout.TableLayout;

/**
 * Panel that displays HepRep object visibility trees and related controls.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: TreePanel.java 13833 2011-05-10 23:26:00Z onoprien $
 */
public class TreePanel extends ControlPanel {

// -- Private parts : ----------------------------------------------------------

  private HepRepPanelModel _plotModel;

  private HepRepTree _typeTree;
  private HepRepTree _instanceTree;

  private JSplitPane _treeSplitPanel;

  private JCheckBox _applyImmediately;
  private JButton _apply;

  private JCheckBox _hideType;
  private JSpinner _hideTypeLevel;
  private JCheckBox _hideInstance;
  private JSpinner _hideInstanceLevel;

  private ChangeListener _modelListener;

// -- Construction : -----------------------------------------------------------

  public TreePanel() {

    super("Visibility Tree", "Set or Clear Visibility of Objects", WiredBaseImage.getIcon("Tree%w", 16), null);
    setLayout(new TableLayout());

    // Types tree :

    _typeTree = new HepRepTree(null);

    // Instances tree :

    _instanceTree = new HepRepTree(null) {
      public String getToolTipText(MouseEvent evt) {
        try {
          if (getRowForLocation(evt.getX(), evt.getY()) == -1) {
            return null;
          }
          TreePath curPath = getPathForLocation(evt.getX(), evt.getY());
          DefaultMutableTreeNode last = (DefaultMutableTreeNode) curPath.getLastPathComponent();
          HepRepInstance instance = (HepRepInstance) last.getUserObject();
          Set attValues = instance.getAttValuesFromNode();
          if (attValues.isEmpty()) {
            return null;
          }
          StringBuffer builder = new StringBuffer("<html><table><tr><th>Name</th><th>Value</th>");
          for (Iterator i = attValues.iterator(); i.hasNext();) {
            HepRepAttValue value = (HepRepAttValue) i.next();
            builder.append("<tr><td>").append(value.getName()).append("</td><td>").append(value.getAsString()).append("</td></tr>");
          }
          builder.append("</table>");
          return builder.toString();
        } catch (Exception x) {
          return null;
        }
      }
    };
    _instanceTree.setToolTipText("");

    String insets = " [3 3 3 3] ";

    _treeSplitPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    _treeSplitPanel.setDividerLocation(200);
    add(_treeSplitPanel, "0 1 2 1" + insets + "wh");

    JPanel typeTreePane = new JPanel();
    typeTreePane.setLayout(new BorderLayout());
    typeTreePane.add(new JLabel("Types"), BorderLayout.NORTH);
    typeTreePane.add(new JScrollPane(_typeTree,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
    _treeSplitPanel.add(typeTreePane);

    JPanel instanceTreePane = new JPanel();
    instanceTreePane.setLayout(new BorderLayout());
    instanceTreePane.add(new JLabel("Instances"), BorderLayout.NORTH);
    instanceTreePane.add(new JScrollPane(_instanceTree,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
    _treeSplitPanel.add(instanceTreePane);

    // Apply, Apply Immediately controls :

    _applyImmediately = new JCheckBox("Apply immediately");
    add(_applyImmediately, "0 2" + insets + "w");
    _applyImmediately.addPropertyChangeListener("enabled", new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        _apply.setEnabled(_applyImmediately.isEnabled() && !_applyImmediately.isSelected());
      }
    });
    _applyImmediately.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        boolean selected = _applyImmediately.isSelected();
        _apply.setEnabled(!selected);
        if (_plotModel != null) _plotModel.setApplyImmediately(selected);
      }
    });

    _apply = new JButton("Apply");
    _apply.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {_plotModel.fireEvent(this);}
    });
    add(_apply, "1 2" + insets + "wr");

    // Hide types controls :

    _hideType = new JCheckBox("Hide Types below level: ");
    add(_hideType, "0 3" + insets + "w");
    _hideType.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        boolean selected = _hideType.isSelected();
        _hideTypeLevel.setEnabled(selected);
        if (_plotModel != null) _plotModel.setTypesDepth(selected);
      }
    });
    _hideType.addPropertyChangeListener("enabled", new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        _hideTypeLevel.setEnabled(_hideType.isEnabled() && _hideType.isSelected());
      }
    });

    _hideTypeLevel = new JSpinner(new SpinnerNumberModel(0,0,0,1));
    _hideTypeLevel.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent event) {
        if (_plotModel != null && isEnabled()) {
          _plotModel.setTypesDepth(((Number)(_hideTypeLevel.getValue())).intValue());
        }
      }
    });

    add(_hideTypeLevel, "1 3" + insets + "wr");
    _hideTypeLevel.setEnabled(false);

    // Hide instances controls :

    _hideInstance = new JCheckBox("Hide Instances below level: ");
    add(_hideInstance, "0 4" + insets + "w");
    _hideInstance.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        boolean selected = _hideInstance.isSelected();
        _hideInstanceLevel.setEnabled(selected);
        if (_plotModel != null) _plotModel.setInstancesDepth(selected);
      }
    });
    _hideInstance.addPropertyChangeListener("enabled", new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        _hideInstanceLevel.setEnabled(_hideInstance.isEnabled() && _hideInstance.isSelected());
      }
    });

    _hideInstanceLevel = new JSpinner(new SpinnerNumberModel(0,0,0,1));
    _hideInstanceLevel.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent event) {
        if (_plotModel != null && isEnabled()) {
          _plotModel.setInstancesDepth(((Number)(_hideInstanceLevel.getValue())).intValue());
        }
      }
    });

    add(_hideInstanceLevel, "1 4" + insets + "wr");
    _hideInstanceLevel.setEnabled(false);

    _modelListener = new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        if (!isSelected() || e.getSource() == this) return;
        _instanceTree.repaint();
      }
    };
  }

// -- Attaching the panel to a WiredPlot : -------------------------------------

  /** Called when this panel is selected or deselected. */
  public void setSelected(boolean selected) {
    super.setSelected(selected);
    if (!selected) return;

    if (_plotModel != null) _plotModel.removeListener(_modelListener);

    WiredPlot plot = WiredPlugin.getPlugin().getPlot();
    GraphicsPanel panel = (plot == null ? null : plot.getGraphicsPanel());
    HepRepPanel heprepPanel = (panel instanceof HepRepPanel ? (HepRepPanel) panel : null);

    setEnabled(false);
    if (heprepPanel != null) {

      _plotModel = heprepPanel.getModel();
      _typeTree.setModel(_plotModel.getTypesModel());
      _instanceTree.setModel(_plotModel.getInstancesModel(), false);

      boolean state = _plotModel.isAppliedImmediately();
      _applyImmediately.setSelected(state);
      _apply.setEnabled(!state);

      state = _plotModel.isTypesDepthLimited();
      _hideType.setSelected(state);
      SpinnerNumberModel sm = (SpinnerNumberModel) _hideTypeLevel.getModel();
      sm.setMaximum(_plotModel.getTypesMaxDepth());
      sm.setValue(_plotModel.getTypesDepth());
      _hideTypeLevel.setEnabled(state);

      state = _plotModel.isInstancesDepthLimited();
      _hideInstance.setSelected(state);
      sm = (SpinnerNumberModel) _hideInstanceLevel.getModel();
      sm.setMaximum(_plotModel.getInstancesMaxDepth());
      sm.setValue(_plotModel.getInstancesDepth());
      _hideInstanceLevel.setEnabled(state);

      setEnabled(true);
      _plotModel.addListener(_modelListener, HepRepPanelModel.Event.Type.SELECTION);

    } else { // setting null plot

      _plotModel = null;
      _typeTree.setModel(null);
      _instanceTree.setModel(null);
      setEnabled(false);

    }
  }

  public void setEnabled(boolean state) {
    super.setEnabled(state);
    _applyImmediately.setEnabled(state);
    _hideType.setEnabled(state);
    _hideInstance.setEnabled(state);
  }
}
